# CMake command line options
# --------------------------

include_guard(GLOBAL)

set(CMAKE_VERBOSE_MAKEFILE OFF CACHE BOOL "OFF")

option(BUILD_SHARED_LIBS "Build shared libraries" ON)
option(ADDRESS_SANITIZER "Use address sanitizer in the build" OFF)
option(THREAD_SANITIZER "Use thread sanitizer in the build" OFF)
option(UNDEFINED_SANITIZER "Use undefined behavior sanitizer in the build" OFF)
if (ADDRESS_SANITIZER AND THREAD_SANITIZER)
  message(FATAL_ERROR "Can only supply one of ADDRESS_SANITIZER and THREAD_SANITIZER")
endif()

set(MIN_LOG_LEVEL "DEBUG" CACHE STRING "Minimum logging level: TRACE,DEBUG,INFO,WARN,ERROR,CRITICAL,OFF")

# If we are a single-config build (make, Ninja), set default to Release
# Multi-config generators are the IDE-based ones, like VS projects. We don't
# support those (yet), but they may work YMMV.
get_property(IS_MULTI_CONFIG GLOBAL PROPERTY GENERATOR_IS_MULTI_CONFIG)
if (NOT IS_MULTI_CONFIG AND NOT DEFINED CMAKE_BUILD_TYPE)
  message(STATUS "Setting build type to 'Release' as none was specified.")
  set(CMAKE_BUILD_TYPE Release CACHE STRING "Choose the type of build." FORCE)

  # Set the possible values of build type for cmake-gui, ccmake
  set_property(CACHE CMAKE_BUILD_TYPE PROPERTY STRINGS "Debug" "Release" "MinSizeRel" "RelWithDebInfo")
endif()

message(STATUS "Build type is: ${CMAKE_BUILD_TYPE}")

# Look for a compiler cache (ccache on Mac/Linux, buildcache on Windows)
if (NOT WIN32)
  find_program(CCACHE ccache)
  if (CCACHE)
    message(STATUS "Using ccache")
    set(CMAKE_CXX_COMPILER_LAUNCHER ${CCACHE})
  else()
    message(STATUS "Did not find ccache - consider installing it if you can")
  endif()
else ()
  find_program(BUILDCACHE buildcache)
  if (BUILDCACHE)
    message(STATUS "Using buildcache")
    set_property(GLOBAL PROPERTY RULE_LAUNCH_COMPILE "${BUILDCACHE}")
  else()
    message(STATUS "Did not find buildcache - consider installing it")
  endif()
endif()

# Generate compile_commands.json to make it easier to work with clang based
# tools. If you want clangd to pick this stuff up, symlink this file (in the
# build directory) into the root directory of your source tree.
set(CMAKE_EXPORT_COMPILE_COMMANDS ON)

# Prevent in-source builds
if (${CMAKE_SOURCE_DIR} STREQUAL ${CMAKE_BINARY_DIR})
  message(FATAL_ERROR "In-source builds not allowed")
endif()
