include_guard(GLOBAL)

# Add a Googletest-based unit test
#
# Arguments:
#  TEST_NAME  Name of the test executable
#  SOURCES    List of source files to compile into the test
#  ARGUMENTS  Any additional flags that need to be added to the command
function(d5_sdk_add_gtest TEST_NAME)
  if(TARGET GTest::gtest AND TARGET GTest::gtest_main)
    set(options)
    set(oneValueArgs)
    set(multiValueArgs SOURCES ARGUMENTS)
    cmake_parse_arguments(d5_sdk_add_gtest "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

    add_executable(${TEST_NAME} ${d5_sdk_add_gtest_SOURCES})
    target_link_libraries(${TEST_NAME}
      compiler_flags
      GTest::gtest
      GTest::gtest_main)
    if(NOT WIN32)
      if(NOT TARGET Threads::Threads)
        find_package(Threads REQUIRED)
      endif()
      target_link_libraries(${TEST_NAME} Threads::Threads)
    endif()

    set(cmd_args)
    if(d5_sdk_add_gtest_ARGUMENTS)
      set(cmd_args ${cmd_args} ${d5_sdk_add_gtest_ARGUMENTS})
    endif()
    add_test(NAME ${TEST_NAME} COMMAND ${TEST_NAME} ${cmd_args})

    if(WIN32)
      set_tests_properties(${TEST_NAME} PROPERTIES WORKING_DIRECTORY "${CMAKE_BINARY_DIR}\\bin")
    endif()
  else()
    message(WARNING "Missing GTest! Skipping test ${TEST_NAME} ...")
  endif()
endfunction()
