/** Interface file for the ephemeris API
 *
 *  \file GetEphemerisInterface.h
 */

#pragma once

namespace d5::apis {

/// \cond HIDDEN_SYMBOLS
namespace plugins {
class EphemerisInterface;
}
/// \endcond

/** Provides an API for getting the ephemeris plugin for the simulation.
 *
 *  This ephemeris getter wraps around the plugin of a similar name
 *  (plugins::EphemerisInterface). Its main function is to act as a gateway to
 *  that plugin through the plugin Resource mechanism that is provided to
 *  plugins at runtime (setup). Since ephemeris is itself a plugin, it cannot be
 *  used until setup is complete and this wrapper will issue an error if an
 *  attempt is made to access it before the plugin is available (plugins can
 *  retain a shared pointer to this resource to use it later, e.g. during init).
 *
 *  \ingroup apis
 */
class GetEphemerisInterface {
public:
  /// Destructor
  virtual ~GetEphemerisInterface() = default;

  /** Request the simulation ephemeris model with interface defined by
   *  plugins::EphemerisInterface. If the ephemeris model is available
   *  (any time after simulation setup is complete), the pointer is valid.
   *  Otherwise an error will be thrown.
   */
  virtual plugins::EphemerisInterface *getEphemerisModel() = 0;
};
} // namespace d5::apis
