/** Interface file for the Weather API
 *
 *  \file GetWeatherInterface.h
 */

#pragma once

namespace d5::apis {

/// \cond HIDDEN_SYMBOLS
namespace plugins {
class WeatherInterface;
}
/// \endcond

/** Provides an API for getting the Weather plugin for the simulation.
 *
 *  This weather getter wraps around the plugin of a similar name
 *  (plugins::WeatherInterface). Its main function is to act as a gateway to
 *  that plugin through the plugin Resource mechanism that is provided to
 *  plugins at runtime (setup). Since weather is itself a plugin, it cannot be
 *  used until setup is complete and this wrapper will issue an error if an
 *  attempt is made to access it before the plugin is available (plugins can
 *  retain a shared pointer to this resource to use it later, e.g. during init).
 *
 *  \ingroup apis
 */
class GetWeatherInterface {
public:
  /// Destructor
  virtual ~GetWeatherInterface() = default;

  /** Request the simulation weather model with interface defined by
   *  plugins::WeatherInterface. If the weather model is available
   *  (any time after simulation setup is complete), the pointer is valid.
   *  Otherwise an error will be thrown.
   */
  virtual plugins::WeatherInterface *getWeatherModel() = 0;
};
} // namespace d5::apis
