/** Interface file for DIRSIG's indexing resource
 *
 *  \file IndexInterface.h
 */

#pragma once

namespace d5::apis {

/// \addtogroup apis
/// \{

/** Communicates the current schedule indexing to a plugin.
 *
 * This resource provides a way for plugins to find out the current context of
 * a call being made relative to the scheduled events of a simulation. The index
 * object gives information about the current event index (as well as the
 * total number of events) as well as the current sensor, task and capture
 * indexes.
 */
class IndexInterface {
public:
  /// Constructor
  IndexInterface() = default;

  /// Destructor
  virtual ~IndexInterface() = default;

  struct CurrentIndexes {
    size_t eventIndex =
        0; // index into all events (e.g. captures) of a simulation
    size_t eventCount = 0; // total number of events in a simulation
    size_t sensorIndex =
        0; // internal index of the current sensor being worked on
    size_t taskIndex = 0;    // sensor-specific task index
    size_t captureIndex = 0; // sensor-specific capture index;
  };

  /// Get the current index object, called by the plugin to obtain current
  /// indexes
  virtual const CurrentIndexes &getIndexes() const = 0;
};

/// \}
} // namespace d5::apis
