#pragma once

#include "dirsig5/base/Geometry.h"
#include "dirsig5/base/IntID.h"
#include "dirsig5/base/SpectralVector.h"

namespace d5::apis {

class ObjectInterface;

struct Node {
  /// Actual location of the node.
  Point pathPoint = {0, 0, 0};

  /// Origin of the next ray
  Point rayOrigin = {0, 0, 0};

  /// Direction of the next ray
  Vector rayDirection = {0, 0, 0};

  /// Distance traveled along the path.
  double pathDist = 0;

  /// Distance traveled to a hit
  double hitDist = 0;

  /// Elapsed time to the node along path
  double elapsedTime = 0;

  /// Direction of the incident ray.
  Vector exitantDirection = {0, 0, 0};

  /// Exitant vector in local coords
  Vector localExitant = {0, 0, 0};

  /// Global normal vector at the hit
  Vector normal = {0, 0, 0};

  /// Global geometric normal vector at the hit
  Vector Ng = {0, 0, 0};

  /// Flag indicating coordinates are in ECEF
  bool usingECEF = false;

  /// Weight associated with a sample
  SpectralVector sampleWeight{};

  /// Localization of the ray
  IntID sceneID = INVALID_ID;

  /// Original proxy scene ID (if applicable)
  IntID proxySceneID = INVALID_ID;

  /// Surface optical property ID.
  IntID opticalPropID = INVALID_ID;

  /// Temperature model ID
  IntID tempModelID = INVALID_ID;

  /// Material used for truth (no mappings)
  IntID truthMatID = INVALID_ID;

  /// Instance ID used for truth
  IntID truthInstID = INVALID_ID;

  /// Geometry ID
  IntID geomID = INVALID_ID;

  /// Instance ID
  IntID instID = INVALID_ID;

  /// Primitive ID
  IntID primID = INVALID_ID;

  /// The hit object
  const ObjectInterface *hitObject{};

  /// Temperature at the node [K]
  double temperature = 0;

  /// Barycentric coordinate of hit
  double bary_u = 0;

  /// Barycentric coordinate of hit
  double bary_v = 0;

  /// Cosine of view angle to surface
  double NdotV = 0;

  /// U texture coordinate
  double U = 0;

  /// V texture coordinate
  double V = 0;

  /// Path radiance for reference wavelength
  double pathRadiance = 0;

  /// Path transmission for reference wavelength
  double pathTransmission = 1;

  /// Re-usable space transform from global to local
  Eigen::Quaterniond spaceTransform = {1, 0, 0, 0};

  /// Random seed
  std::uint32_t seed = 0;
};

} // namespace d5::apis
