/** Interface file for PlatformInfo
 *
 *  \file PlatformInfo.h
 */

#pragma once

#include "dirsig5/base/DualQuaternion.h"
#include "dirsig5/base/Geometry.h"
#include "dirsig5/base/IntID.h"
#include "dirsig5/base/Json.h"

namespace d5::apis {

/// \addtogroup apis
/// \{

/** Information about what a capture looks like, including a basic description
 *  of the field-of-view (FOV).
 */
struct PlatformInfo {
  /// The scene index for translating local coords (assume ECEF if < 0)
  IntID sceneIndex = 0;

  /// The nominal sensor position/orientation (local/ECEF coords)
  DualQuaternion sensorDualQuaternion;

  /// The nominal velocity vector with magnitude (local/ECEF coords)
  Vector sensorVelocity;

  /// The angular FOV in the X direction [radians]
  double sensorFovX = 0;

  /// The angular FOV in the Y direction [radians]
  double sensorFovY = 0;

  /// The nominal vehicle position/orientation (local/ECEF coords)
  DualQuaternion vehicleDualQuaternion;
};

D5_TO_JSON(PlatformInfo, j, i) {
  j = Json{{"scene_index", i.sceneIndex},
           {"sensor_dual_quaternion", i.sensorDualQuaternion},
           {"sensor_velocity", i.sensorVelocity},
           {"sensor_fov_x", i.sensorFovX},
           {"sensor_fov_y", i.sensorFovY},
           {"vehicle_dual_quaternion", i.vehicleDualQuaternion}};
}

D5_FROM_JSON(PlatformInfo, j, i) {
  j.at("scene_index").get_to(i.sceneIndex);
  j.at("sensor_dual_quaternion").get_to(i.sensorDualQuaternion);
  j.at("sensor_velocity").get_to(i.sensorVelocity);
  j.at("sensor_fov_x").get_to(i.sensorFovX);
  j.at("sensor_fov_y").get_to(i.sensorFovY);
  j.at("vehicle_dual_quaternion").get_to(i.vehicleDualQuaternion);
}

/// \}
} // namespace d5::apis
