/** Interface file for the scene intersection API
 *
 *  \file SceneIntersectionInterface.h
 */

#pragma once

namespace d5::apis {

// prototype the intersection info
struct IntersectInfo;

/** The scene intersection interface allows plugins to directly intersect
 *  a ray with a single scene in the simulation.
 *
 *  \ingroup apis
 */
class SceneIntersectionInterface {
public:
  virtual ~SceneIntersectionInterface() = default;

  /** Intersect a scene (identified by the given index into user scenes).
   *  The data interchange is done using the standard "IntersectInfo"
   *  struct as defined in d5::apis::Plugin. The function will return
   *  true if something was hit, and false otherwise to eliminate a check.
   *  Ray are assumed to be defined in the same coordinate system as the
   *  requested scene.
   */
  virtual bool intersectScene(size_t sceneIndex,
                              d5::apis::IntersectInfo &info) = 0;
};
} // namespace d5::apis
