/** Interface file for the source API
 *
 *  \file SourceInfo.h
 */

#pragma once

#include "dirsig5/base/Geometry.h"
#include "dirsig5/base/IntID.h"
#include "dirsig5/base/SpectralVector.h"

namespace d5::apis {

/** This is a simple wrapper around a source meant to provide any necessary
 *  info to a plugin (for instance, incorporating a direct contribution).
 *
 *  \ingroup apis
 */
class SourceInfo {
public:
  enum Type { POINT, DISK };

  struct PointData {
    Point ecef_center;
    Vector ecef_pointing;
    size_t sceneIndex;
    size_t sourceIndex;
    SpectralVector irradiance;
    std::string materialID;
    std::string materialName;
  };

  struct DiskData {
    Vector toSource;
    double tanHalfAngle;
    size_t sourceIndex;
    SpectralVector irradiance;
  };

  virtual ~SourceInfo() = default;

  [[nodiscard]] virtual Type getType() const = 0;

  /// Get the state of the source at the given time as a type specific
  /// set of parameters. These may include the direction to the source,
  /// its distance, the pointing direction of the source, its irradiance,
  /// and the tangent of the half angle of the width of the source
  ///
  ///\{

  virtual void getPointState(float time, const Point &viewOrigin,
                             PointData &params) const = 0;

  virtual void getDiskState(float time, const Point &viewOrigin,
                            DiskData &params) const = 0;

  virtual void getMetaInfo(std::string &id, std::string &name) const = 0;

  ///\}
};
} // namespace d5::apis
