/** Interface file for the TimeWindow API
 *
 *  \file TimeWindow.h
 */

#pragma once

#include "dirsig5/base/Error.h"
#include "dirsig5/base/Json.h"

namespace d5::apis {

/// \addtogroup apis
/// \{

/** Defines a time window (start and end time) measured from
 *  the reference date/time.
 */
struct TimeWindow {
  TimeWindow() = default;

  /// Constructor with valid times.
  TimeWindow(double start, double end)
      : startTime(start), endTime(end), valid(true) {}

  /// Sets a valid time window.
  void setWindow(double start, double end) {
    startTime = start;
    endTime = end;
    DIRSIG_THROW_IF(endTime < startTime,
                    "Attempted to create an invalid time window!");
    valid = true;
  }

  /// Resets the valid flag back to false.
  void reset() { valid = false; }

  /// The elapsed time [seconds] for the start of the window
  double startTime = 0;

  /// The elapsed time [seconds] for the end of the window
  double endTime = 0;

  /// If false, the plugin will no longer be queried for tasks/captures
  bool valid = false;
};

D5_TO_JSON(TimeWindow, j, t) {
  if (t.valid) {
    j = Json::array({t.startTime, t.endTime});
  } else {
    j = nullptr;
  }
}

D5_FROM_JSON(TimeWindow, j, t) {
  t.reset();
  if (j.is_array()) {
    double start = 0, end = 0;
    j.at(0).get_to(start);
    j.at(1).get_to(end);
    t.setWindow(start, end);
  }
}

/// \}
} // namespace d5::apis
