/** Interface file for DynamicInstanceInterface
 *
 *  \file DynamicInstanceInterface.h
 */

#pragma once

#include "dirsig5/apis/ObjectInterface.h"
#include "dirsig5/apis/Plugin.h"

namespace d5::apis::plugins {

/** \defgroup plugin_apis_dyinst Dynamic Instance Plugin
 *  \ingroup plugin_apis
 *  Holds the plugin API for a dynamic instance plugin
 */

/// \addtogroup plugin_apis_dyinst
/// \{

/// Definition of input/output variables for the init interface
namespace DynamicInstanceInit {

struct In {
  /// Instance definition, usually a payload (note that this
  /// functionality mimics plugin setup, but this plugin does not
  /// provide a setup implementation).
  void *definition = nullptr;

  /// Scene origin (latitude, longitude, altitude in [deg] and [m])
  Pointf lla;
};

struct Out {
  /// Transform callback. Provides a dual quaternion-based
  /// transformation for the given object/instance indexes and time.
  DynamicCallback callback;
};
} // namespace DynamicInstanceInit

class DynamicInstanceInterface : virtual public Plugin {
public:
  static void getPluginTypeName(std::vector<std::string> &typeNames) {
    typeNames.push_back("dynamicinstance");
  }

  /** Setup
   *
   *  This does not use a setup, so make sure it never gets
   *  implemented in an inherited object.
   */
  Status Setup(const Setup::In & /*in*/, Setup::Out & /*out*/) {
    // do nothing
    return SUCCESS;
  }

  virtual Status init(const DynamicInstanceInit::In &in,
                      DynamicInstanceInit::Out &out) = 0;
};

/// \}
} // namespace d5::apis::plugins
