#pragma once

#include <string>
#include <variant>
#include <vector>

#include "dirsig5/atmosphere/backends/ModtranCommon.h"

namespace d5::atm::modtran::json {

/// Filename of a JSON file
struct JsonFilename {
  std::string filename;
};

/// Contents of a JSON file
struct JsonContents {
  std::string contents;
};

/// Either a filename, file contents or none
using JsonVariant = std::variant<std::nullptr_t, JsonFilename, JsonContents>;

/// Configuration object for the MODTRAN backend
struct Config {
  /// Either a profile name or a profile itself
  std::variant<std::string, Profile> profile;

  /// Any command-line options to pass to MODTRAN
  std::vector<std::string> modtranOptions;

  /// Filename(s) or content(s) of the default tape5 template
  JsonVariant json;

  /// Whether to extract atmospheric profiles
  bool extractProfile = false;
};

struct RequestBackendInput {
  /// Filename(s) or content(s) of the tape5 template
  JsonVariant json;
};

} // namespace d5::atm::modtran::json
