#pragma once

#include <string>
#include <variant>
#include <vector>

#include "dirsig5/atmosphere/backends/ModtranCommon.h"

namespace d5::atm::modtran::tape {

/// Filename of a Tape5 file
struct Tape5Filename {
  std::string filename;
};

/// Contents of a Tape5 file
struct Tape5Contents {
  std::string contents;
};

/// Either a filename, file contents or none
using Tape5Variant =
    std::variant<std::nullptr_t, Tape5Filename, Tape5Contents, Tape5Parameters>;

/// Configuration object for the MODTRAN backend
struct Config {
  /// Either a profile name or a profile itself
  std::variant<std::string, Profile> profile;

  /// Any command-line options to pass to MODTRAN
  std::vector<std::string> modtranOptions;

  /// Filename(s) or content(s) of the default tape5 template
  Tape5Variant tape5;

  /// Optional modroot filename
  std::string modRootFilename;

  /// Whether to extract atmospheric profiles
  bool extractProfile = false;
};

struct RequestBackendInput {
  /// Filename, content or parameters of the tape5 template
  Tape5Variant tape5;
};

} // namespace d5::atm::modtran::tape
