/** Common header for ID's
 *
 *  \file IntID.h
 */

#pragma once

#include <cstddef>
#include <cstdint>
#include <stack>

namespace d5 {

/// \addtogroup common
/// \{

using IntID = std::int32_t;

using IntIDStack = std::stack<IntID>;

inline constexpr IntID INVALID_ID = -1;

// Earth model scene ID
inline constexpr IntID EarthSceneID = 0;

// Null material ID
constexpr IntID NullMatID = 0;

// Null property ID
constexpr IntID NullPropID = INVALID_ID;

// Core Earth backing material
inline constexpr IntID CoreEarthMaterial = NullMatID;

// Geometry IDs for the global objects
//@{
inline constexpr IntID CoreGeometryID = 0;
inline constexpr IntID EarthGeometryID = 1;
inline constexpr IntID CloudGeometryID = 2;
//@}

// ID for the default medium (atmosphere)
inline constexpr IntID DefaultMedium = 0;

/// \}

} // namespace d5
