#pragma once

#include <string>
#include <unordered_map>

#include "dirsig5/base/Export.h"

namespace d5::StandardPaths {

/// The user home directory.
///
/// This obtains the absolute path to the user home directory by reading
/// the relevant environment variable. On Linux and Mac, this is `HOME`.
/// On Windows, this is `USERPROFILE`.
///
D5_BASE_EXPORT std::string Home();

/// The user documents directory.
///
/// This obtains the absolute path to the user documents directory
/// by appending `/Documents` to the user home directory.
///
D5_BASE_EXPORT std::string Documents();

/// The DIRSIG home/install directory.
D5_BASE_EXPORT std::string DirsigHome();

/// The DIRSIG executable directory.
///
D5_BASE_EXPORT std::string DirsigExecutables();

/// The DIRSIG plugin directory.
///
/// \param[in] plugin
/// The (optional) plugin subdirectory name.
///
D5_BASE_EXPORT std::string DirsigPlugins(const std::string &plugin = {});

/// The user plugin directory.
///
/// \param[in] plugin
/// The (optional) plugin subdirectory name.
///
D5_BASE_EXPORT std::string UserPlugins(const std::string &plugin = {});

/// The DIRSIG settings INI filename.
D5_BASE_EXPORT std::string DirsigIniFilename();

/// Helper to resolve path replacements.
///
/// \param[in] path       The path to resolve.
/// \param[in] what       The lookup table for replacements.
/// \param[in] useSysEnv  Use system environment as well?
///
/// This helper resolves path replacements like `$REPLACE_ME` or
/// `${REPLACE_ME}`. The main purpose of this function is to resolve
/// `${DIRSIG_HOME}` without it being present in the system environment,
/// but also to resolve other system environment variables, like `$HOME`
/// or `$PATH`. Client code can also supply additional replacements
/// via the `what` parameter, and can optionally disable system environment
/// checking by passing false for `useSystemEnv`.
///
D5_BASE_EXPORT std::string
withReplacements(const std::string &path,
                 const std::unordered_map<std::string, std::string> &what = {},
                 bool useSysEnv = true);

/// Helper to convert to the native system separator (backslash on Windows).
D5_BASE_EXPORT std::string withNativeSeparators(const std::string &path);

} // namespace d5::StandardPaths
