#pragma once

#include <utility>
#include <vector>

#include "dirsig5/base/Export.h"
#include "dirsig5/base/Truth.h"

namespace d5 {

/// \addtogroup utility
/// \{

/** Takes a name and adds one or more truth collectors to the supplied vector.
 *
 *  \param[in] name           Name of the truth collector
 *  \param[in] params         Parameters for the truth collector
 *  \param[out] truthRequest  Vector onto which to add requests
 *
 *  \return true if any collectors were added, false otherwise.
 */
D5_BASE_EXPORT bool
AddTruthCollector(const std::string &name,
                  const std::vector<d5::Truth::Parameter> &params,
                  std::vector<d5::Truth> &truthRequests);

/** Creates the band name for the given truth collector
 *
 *  \param truth  The truth collector
 *
 *  \return The band name in a truth image
 *  \throw std::exception subclass on any error
 */
D5_BASE_EXPORT std::string TruthBandName(const d5::Truth &truth);

/// \}
} // namespace d5
