#pragma once

#include <utility>

namespace d5 {

/** Version information (a major and minor ID).
 *  \ingroup common
 */
struct Version {
  constexpr Version() noexcept = default;

  /// Construct from major and minor IDs.
  constexpr Version(int maj, int min) noexcept : majorID(maj), minorID(min) {}

  /// Get major and minor IDs as an STL pair.
  [[nodiscard]] constexpr std::pair<int, int> asPair() const noexcept {
    return {majorID, minorID};
  }

  /// \name Comparison operators
  /// \{

  constexpr bool operator==(const Version &rhs) const noexcept {
    return asPair() == rhs.asPair();
  }

  constexpr bool operator!=(const Version &rhs) const noexcept {
    return asPair() != rhs.asPair();
  }

  constexpr bool operator>=(const Version &rhs) const noexcept {
    return asPair() >= rhs.asPair();
  }

  constexpr bool operator<=(const Version &rhs) const noexcept {
    return asPair() <= rhs.asPair();
  }

  constexpr bool operator>(const Version &rhs) const noexcept {
    return asPair() > rhs.asPair();
  }

  constexpr bool operator<(const Version &rhs) const noexcept {
    return asPair() < rhs.asPair();
  }

  /// \}

  /// The major version ID.
  int majorID = 1;

  /// The minor version ID.
  int minorID = 0;
};

} // namespace d5
