#pragma once

#include <algorithm>

#include "dirsig5/brdf/BrdfModel.h"

namespace d5::brdf {

class D5_BRDF_EXPORT Ward final : public BrdfModel {
public:
  struct Parameters {
    struct SpectralParameters {
      /// Wavelength in micrometers.
      double wavelengthMicrons = 0;

      /// Roughness coefficient.
      double sigma = 0.5;

      /// Diffuse coefficient.
      double diffuse = 0.7;

      /// Specular coefficient.
      double specular = 0.3;
    };

    std::vector<SpectralParameters> spectral;
  };

  Ward(std::vector<double> wavelengthMicrons, Parameters params)
      : BrdfModel(std::move(wavelengthMicrons)), params_(std::move(params)) {
    std::sort(
      params_.spectral.begin(), params_.spectral.end(),
      [](const auto& lhs, const auto& rhs) {
        return lhs.wavelengthMicrons < rhs.wavelengthMicrons;
      });
    init();
  }

  [[nodiscard]] const Parameters& parameters() const { return params_; }

  [[nodiscard]] std::vector<double> sample(
    double illumZenithRad,
    double illumAzimuthRad,
    double viewZenithRad,
    double viewAzimuthRad) const override;

private:
  void init() override;

  [[nodiscard]] std::vector<double> doSimplifySpectralBrdf(
    std::vector<double>& nadirDHR) override;

  void doGetHashInputs(const HashCallback& callback) const override;

private:
  Parameters params_;

  std::vector<double> sigma_;

  std::vector<double> diffuse_;

  std::vector<double> specular_;
};

}  // namespace d5::brdf
