#pragma once

#include <functional>

#include "dirsig5/brdf/Export.h"

namespace d5::brdf::detail {

/** An abstraction for a type that is hashable
 *  \ingroup brdf
 */
class D5_BRDF_EXPORT Hashable {
public:
  using HashCallback = std::function<void(const void*, size_t)>;

  /** Gets data buffers to incorporate into a hash that represent the inputs
   *  of the BRDF model.
   *
   *  \param callback  A callback function that will be called with a
   *                   buffer/size with each parameter block for this BRDF model
   */
  virtual void getHashInputs(const HashCallback& callback) const = 0;

protected:
  template <typename T>
  static void AddHashData(const T& data, const HashCallback& callback) {
    callback(&data, sizeof(T));
  }

  template <typename T>
  static void AddHashData(
    const std::vector<T>& data, const HashCallback& callback) {
    callback(data.data(), data.size() * sizeof(T));
  }
};

}  // namespace d5::brdf::detail
