#pragma once

#include <string>
#include <vector>

#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_geometry
/// \{

/// A control table can toggle tagged objects on and off
struct ControlTable {
  /// Filename of the control table file
  std::string filename;

  /// Tags of the objects being controlled by this table
  std::vector<std::string> tagIndex;

  /// Relative times in seconds for each entry in the control table
  std::vector<float> times;

  /// The states (0 = off, 1 = on) for each tag at each time, states for each
  /// time sample are stored sequentially.
  std::vector<float> states;
};

/** Loads a control table from a file
 *
 *  \param filename  Filename from which to load
 *
 *  \return The loaded control table
 *  \throw std::exception subclass on any error
 */
D5_D4LEGACY_EXPORT ControlTable
ReadControlTableFromFile(const std::string& filename);

/// \}
}  // namespace d5::d4legacy
