#pragma once

#include <array>
#include <memory>
#include <optional>
#include <string>
#include <vector>

#include "dirsig5/base/Geometry.h"
#include "dirsig5/d4legacy/Export.h"
#include "dirsig5/d4legacy/geometry/Geometry.h"
#include "dirsig5/d4legacy/geometry/Instance.h"
#include "dirsig5/d4legacy/geometry/Population.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_geometry
/// \{

/// The DIRSIG4 concept of an object
class D5_D4LEGACY_EXPORT Object {
public:
  Object() = default;
  Object(const Object&) = delete;
  Object(Object&&) = default;
  Object& operator=(const Object&) = delete;
  Object& operator=(Object&&) = default;

  void addGeometry(std::shared_ptr<Geometry> geom, double weight = 1) {
    geometries.push_back(std::move(geom));
    weights.push_back(weight);
  }

  void ReadXML(
    const xml::Node& node,
    const geolib::Facade* facade,
    const FileLocator& fileLocator = {},
    const FileLocator& fileLocatorLocal = {});

public:
  size_t lineNumber = 0;
  bool useLocalSearchPaths = false;
  std::string localMaterialsFilename;
  std::vector<std::string> tags;
  std::optional<Population> population;
  std::vector<RandomFill> randomFills;
  std::vector<std::shared_ptr<Geometry>> geometries;
  std::vector<std::shared_ptr<Instance>> instances;
  std::vector<double> weights;
};

/** A geometry list is a collection of objects. It can also be tagged and
 *  include other geometry lists.
 */
class D5_D4LEGACY_EXPORT GeometryList {
public:
  GeometryList() = default;
  GeometryList(const GeometryList&) = delete;
  GeometryList& operator=(const GeometryList&) = delete;
  GeometryList(GeometryList&&) = default;
  GeometryList& operator=(GeometryList&&) = default;

  void append(GeometryList&& other) {
    size_t oldSize = objects.size();
    objects.resize(oldSize + other.objects.size());
    for (size_t i = 0; i < other.objects.size(); i++)
      objects[oldSize + i] = std::move(other.objects[i]);
    includes.reserve(includes.size() + other.includes.size());
    for (auto& f : other.includes) includes.push_back(std::move(f));
  }

  void ReadODBFile(
    const std::string& filename,
    const geolib::Facade* facade,
    const FileLocator& fileLocator = {});

  void ReadXMLFile(
    const std::string& filename,
    const geolib::Facade* facade,
    const FileLocator& fileLocator = {});

  void ReadXML(
    const xml::Node& node,
    const geolib::Facade* facade,
    const FileLocator& fileLocator = {},
    const FileLocator& fileLocatorLocal = {});

public:
  std::vector<std::string> tags;
  std::vector<std::string> includes;
  std::vector<Object> objects;
};

/// \}
}  // namespace d5::d4legacy
