/** Interface file for Geometry::dispatch
 *
 *  \file GeometryDispatch.h
 */

#pragma once

#include <stdexcept>

#ifndef D4LEGACY_INCLUDED_FROM_GEOMETRY_H
#define D4LEGACY_INCLUDED_FROM_GEOMETRY_DISPATCH_H
#include "../Geometry.h"
#endif

#ifndef D4LEGACY_INCLUDED_FROM_PRIMITIVES_H
#define D4LEGACY_INCLUDED_FROM_GEOMETRY_DISPATCH_H
#include "../Primitives.h"
#endif

namespace d5::d4legacy {

/// \cond HIDDEN_SYMBOLS

template <typename Callable, typename... Args>
decltype(auto) Geometry::dispatch(Callable&& callback, Args&&... args) const {
  switch (type_) {
    case Type::GdbFacetObject:
      return callback(as<GdbFacetObject>(), std::forward<Args>(args)...);
    case Type::ObjFacetObject:
      return callback(as<ObjFacetObject>(), std::forward<Args>(args)...);
    case Type::TdfFacetObject:
      return callback(as<TdfFacetObject>(), std::forward<Args>(args)...);
    case Type::HdfFacetObject:
      return callback(as<HdfFacetObject>(), std::forward<Args>(args)...);
    case Type::OdbInclude:
      return callback(as<OdbInclude>(), std::forward<Args>(args)...);
    case Type::GlistInclude:
      return callback(as<GlistInclude>(), std::forward<Args>(args)...);
    case Type::Box:
      return callback(as<Box>(), std::forward<Args>(args)...);
    case Type::CurvedFrustum:
      return callback(as<CurvedFrustum>(), std::forward<Args>(args)...);
    case Type::Cylinder:
      return callback(as<Cylinder>(), std::forward<Args>(args)...);
    case Type::Sphere:
      return callback(as<Sphere>(), std::forward<Args>(args)...);
    case Type::Sinusoid:
      return callback(as<Sinusoid>(), std::forward<Args>(args)...);
    case Type::SecchiDisk:
      return callback(as<SecchiDisk>(), std::forward<Args>(args)...);
    case Type::Disk:
      return callback(as<Disk>(), std::forward<Args>(args)...);
    case Type::RegularGrid:
      return callback(as<RegularGrid>(), std::forward<Args>(args)...);
    case Type::SurfacePlane:
      return callback(as<SurfacePlane>(), std::forward<Args>(args)...);
    case Type::CatenaryCurve:
      return callback(as<CatenaryCurve>(), std::forward<Args>(args)...);
    case Type::HeightField:
      return callback(as<HeightField>(), std::forward<Args>(args)...);
    case Type::PointSource:
      return callback(as<PointSource>(), std::forward<Args>(args)...);
    case Type::BezierCurveSet:
      return callback(as<BezierCurveSet>(), std::forward<Args>(args)...);
    case Type::Unknown:
      throw std::runtime_error(
          "Geometry::dispatch() encounted unknown geometry type");
  }
}

/// \endcond
}
