#pragma once

#include "dirsig5/d4legacy/Export.h"
#include "dirsig5/raster/Raster.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_imageio
/// \{

/// Options structure for a raster reading request
class D5_D4LEGACY_EXPORT RasterReadOptions {
public:
  /// Whether to read as a grayscale image
  [[nodiscard]] bool readAsGrayscale() const { return readAsGrayscale_; }

  /// Sets whether to read as grayscale
  RasterReadOptions& withReadAsGrayscale(bool r) {
    readAsGrayscale_ = r;
    return *this;
  }

private:
  bool readAsGrayscale_ = false;
};

/** Reads a data array from a file
 *
 *  \param filename  The filename from which to load
 *  \param options   Options to use for the load
 *
 *  \return The read Raster
 *  \throw A subclass of std::exception on any error
 */
D5_D4LEGACY_EXPORT raster::Raster ReadRasterDataFromFile(
  const char* filename, const RasterReadOptions& options = RasterReadOptions());

/// \}
}  // namespace d5::d4legacy
