/** Interface file for temporal images
 *
 *  \file TemporalImages.h
 */

#pragma once

#include <string>
#include <vector>

#include "dirsig5/base/DateTime.h"
#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_image_io
/// \{

/// An entry in a .til file
struct TimeVaryingImageEntry {
  /// Start time for the entry [s]
  DateTime time;

  /// Filename of the image to use during this entry's lifetime
  std::string filename;
};

/** Loads a .til file
 *
 *  \param fileanme  The filename to load
 *
 *  \return The time-varying image data
 *  \throw std::exception subclass on any error
 */
D5_D4LEGACY_EXPORT std::vector<TimeVaryingImageEntry> ReadTilFile(
  const std::string& filename);

/// \}

}  // namespace d5::d4legacy
