/** Interface file for dealing with absorption files
 *
 *  \file AbsorptionFile.h
 */

#pragma once

#include <string>
#include <vector>

#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_mat
/// \{

/// Holds a spectral absorption curve
struct D5_D4LEGACY_EXPORT AbsorptionData {
  /// The wavenumber samples for the spectrum
  std::vector<double> wavenumber;

  /// The absorption spectrum [1/(ppm * m)]
  std::vector<double> absInvPpmM;

  /** Loads an absorption spectrum from a file
   *
   *  \param filename  The filename to load
   *
   *  \return The read absorption data
   *  \throw std::exception subclass on any error
   */
  static AbsorptionData ReadFromFile(const std::string& filename);
};

/// \}
}  // namespace d5::d4legacy
