/** Interface file for EmissivityData
 *
 * \file EmissivityData.h
 */
#pragma once

#include <string>
#include <vector>

#include "dirsig5/base/span.h"
#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_mat
/// \{

/** Emissivity data.
 *
 *  This consists of angular weightings and a number of spectral curves.
 */
class D5_D4LEGACY_EXPORT EmissivityData {
public:
  /// Container to hold the information in a single spectral curve
  struct Curve {
    std::vector<double> wavelengthMicrons;
    std::vector<double> reflectance;
  };

  /// The spectral curves
  std::vector<Curve> curves;

  /// The angular weights
  std::vector<double> angleWeighting;

  /// Average reflectance at given wavelengths
  [[nodiscard]] std::vector<double> averageReflectance(
    const std::vector<double>& wavelengthMicrons) const;
};

/** Loads an ems file from disk
 *
 *  \param[in] filename  The filename from which to read
 *  \param[out] ems      The read emissivity data
 *
 *  \throw std::exception subclass on any error
 */
D5_D4LEGACY_EXPORT void LoadEmsFile(
  const char* filename,
  span<const float> wavelengthsScene,
  EmissivityData& ems);

/// \overload
inline EmissivityData LoadEmsFile(
  const char* filename, span<const float> wavelengthsScene = {}) {
  EmissivityData ems;
  LoadEmsFile(filename, wavelengthsScene, ems);
  return ems;
}

/// \}
}  // namespace d5::d4legacy
