/** Interface file for simple spectral data
 *
 *  \file SimpleSpectralData.h
 */

#pragma once

#include <cstdint>
#include <string>
#include <variant>
#include <vector>

#include "dirsig5/base/span.h"
#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_mat
/// \{

D5_D4LEGACY_EXPORT std::vector<double> ReadSpectralSamples(
  const char* filename);

/// Wrapper struct for simple spectral data (wavelength / value)
struct SimpleSpectralData {
  /// The various supported simple spectral data formats
  enum class Format : uint8_t {
    Unknown = 0,  ///< Format could not be determined
    Text,         ///< DIRISG simple spectral data text format
    XML,          ///< DIRSIG simple spectral data XML format
    SLI           ///< ENVI SLI (Spectral LIbrary)
  };

  std::vector<double> wavelengthsMicron;
  std::vector<double> spectrum;
};

/** Reads simple spectral data from an istream
 *
 *  \param filename     The filename from which to read
 *  \param format       The format of the istream
 *  \param sliIndex     The SLI index, if necessary
 *
 *  \return The read simple spectral data
 *  \throw std::exception subclass if a read erro occurs
 */
D5_D4LEGACY_EXPORT SimpleSpectralData ReadSimpleSpectralDataFromFile(
  const char* filename,
  SimpleSpectralData::Format format,
  const std::variant<int, std::string>& sliIndex = -1,
  span<const float> wavelengthsScene = {});

/// \}
}  // namespace d5::d4legacy
