/** Interface file for the THERM temperature model
 *
 *  \file Therm.h
 */

#pragma once

#include <ostream>

#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_mat
/// \{

/// Material parameters for the THERM temperature model
struct D5_D4LEGACY_EXPORT ThermMaterialProps {
  static constexpr double NotGiven = -1;

  double specificHeat = NotGiven;
  double massDensity = NotGiven;
  double thermalConductivity = NotGiven;
  double exposedArea = NotGiven;
  double solarAbsorption = NotGiven;
  double thermalEmissivity = NotGiven;
  double thickness = 0;
  double selfGeneratedPower = 0;

  inline friend std::ostream& operator<<(
    std::ostream& os, const ThermMaterialProps& p) {
    os << "Specific Heat: " << p.specificHeat
       << "\nMass Density: " << p.massDensity
       << "\nThermal Conductivity: " << p.thermalConductivity
       << "\nExposed Area: " << p.exposedArea << "\n";
    if (p.solarAbsorption != ThermMaterialProps::NotGiven) {
      os << "Solar Absorption: " << p.solarAbsorption << "\n";
    }
    if (p.thermalEmissivity != ThermMaterialProps::NotGiven) {
      os << "Thermal Emissivity: " << p.thermalEmissivity << "\n";
    }
    os << "Thickness: " << p.thickness
       << "\nSelf-generated power: " << p.selfGeneratedPower;
    return os;
  }
};

/// \}
}  // namespace d5::d4legacy
