#pragma once

#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_misc
/// \{

/** Manipulations based on hardware endianness
 *
 *  This class supplies support routines for determining whether the native
 *  architecture is big or little endian.  It also supplies helpful byte
 *  reordering functions to change the endianness for various stored types.
 */
class EndianSupport {
public:
  /// Indicates how multibyte types are stored in memory
  enum ByteOrder {
    UnknownEndian,  ///< Unknown endian storage
    LittleEndian,   ///< Little endian
    BigEndian       ///< Big endian
  };

public:
  /**
   * Determine byte order of native machine.
   *
   * \return   whether native machine stores multibyte types in little or
   *           big endian.
   */
  D5_D4LEGACY_EXPORT static ByteOrder nativeByteOrder();

  /**
   * Swap byte order of two byte type.
   *
   * \param    data      Points to beginning of two byte type to be
   *                     swapped.
   */
  D5_D4LEGACY_EXPORT static void swapTwoByteType(char* data);

  /**
   * Swap byte order of four byte type.
   *
   * \param    data      Points to beginning of four byte type to be
   *                     swapped.
   */
  D5_D4LEGACY_EXPORT static void swapFourByteType(char* data);

  /**
   * Swap byte order of eight byte type.
   *
   * \param    data      Points to beginning of eight byte type to be
   *                     swapped.
   */
  D5_D4LEGACY_EXPORT static void swapEightByteType(char* data);
};

/// \}
}  // namespace d5::d4legacy
