#pragma once

#include <string>

#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_misc
/// \{

/// Measures of distance.
enum class DistanceUnits {
  Inches,       ///< Inches
  Feet,         ///< Feet
  Centimeters,  ///< Centimeters
  Meters        ///< Meters
};

/** Convert string to units.
 *
 *  \param unitString   String declaring units.
 *  \param units        Units parsed from string.
 *
 *  \return true if parse was succesful; false otherwise.
 */
D5_D4LEGACY_EXPORT bool stringToUnits(
  std::string unitString, DistanceUnits& units);

/** Convert units to string
 *
 *  \param units  The units to convert
 *
 *  \return string representation of the units
 */
D5_D4LEGACY_EXPORT std::string to_string(DistanceUnits units);

/** Conversion factor from units to meters.
 *
 *  \param    units        Unit to convert from.
 *
 *  \return   multiplicative conversion factor from units to meters.
 */
D5_D4LEGACY_EXPORT double toMeters(const DistanceUnits& units);

/** Conversion factor from meters to unit.
 *
 *  \param units      Unit to convert to.
 *
 *  \return multiplicative conversion factor from meters to unit.
 */
D5_D4LEGACY_EXPORT double fromMeters(const DistanceUnits& units);

/// Measurements of wavelength
enum class WavelengthUnits {
  Microns,
  Nanometers,
  Millimeters,
  Centimeters,
  Meters,
  Angstroms,
  Wavenumbers,
  GHz,
  MHz
};

/** Convert string to units.
 *
 *  \param unitString   String declaring units.
 *  \param units        Units parsed from string.
 *
 *  \return true if parse was succesful; false otherwise.
 */
D5_D4LEGACY_EXPORT bool stringToUnits(
  const std::string& unitString, WavelengthUnits& units);

/** Convert units to string
 *
 *  \param units  The units to convert
 *
 *  \return string representation of the units
 */
D5_D4LEGACY_EXPORT std::string to_string(WavelengthUnits units);

/** Conversion factor from units to meters.
 *
 *  \param val    Value to convert
 *  \param units  Unit to convert from
 *
 *  \return Value converted to microns
 */
D5_D4LEGACY_EXPORT double toMicrons(double val, const WavelengthUnits& units);

/// \}
}  // namespace d5::d4legacy
