#pragma once

#include <array>
#include <memory>
#include <optional>
#include <vector>

#include "dirsig5/base/DateTime.h"
#include "dirsig5/base/DualQuaternion.h"
#include "dirsig5/d4legacy/Export.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_motion
/// \{

/** Delta motion data model. Delta motion gives position orientation deltas as
 *  a function of time.
 */
struct D5_D4LEGACY_EXPORT DeltaMotion {
  /// Whether the object starts hidden until the first move entry time
  bool startHidden = false;

  /// Whether the object becomes hidden after the last move entry time
  bool endHidden = false;

  /// Defines the initial position of the object
  struct InitialPosition {
    /// Reference date time for the initial position
    std::optional<DateTime> refDateTime;

    /// Initial translation
    std::array<double, 3> translation;

    /// Initial scale
    std::array<double, 3> scale;

    /// Initial rotation (ZYX order)
    std::array<double, 3> rotation;
  };

  /// The initial position of the object
  InitialPosition initialPosition;

  /// Defines a single delta motion record
  struct Move {
    /// Change in time [seconds]
    double dSec = 0;

    /// Change in translation
    std::array<double, 3> dTranslation;

    /// Change in scale
    std::array<double, 3> dScale;

    /// Change in rotation
    std::array<double, 3> dRotation;
  };

  /// The set of delta motion records
  std::vector<Move> moves;

  void ReadMOVFile(const std::string& filename);
};

D5_D4LEGACY_EXPORT std::vector<std::pair<double, DualQuaternion>>
DeltaMotionToDualQuaternions(
  const DeltaMotion& deltaMotion, const std::optional<DateTime>& refDateTime);

/// \}
}  // namespace d5::d4legacy
