#pragma once

#include <array>
#include <memory>
#include <optional>
#include <vector>

#include "dirsig5/base/DateTime.h"
#include "dirsig5/d4legacy/Export.h"
#include "dirsig5/d4legacy/geometry/Rotation.h"
#include "dirsig5/d4legacy/xml/xml.h"
#include "dirsig5/geolib/Location.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_motion
/// \{

/// A data model for DIRSIG4-era generic motion
struct D5_D4LEGACY_EXPORT GenericMotion {
  /// Rotation order
  RotationOrder rotationOrder = RotationOrder::Zyx;

  /// Rotation frame
  enum class RotationFrame { SceneEnu, PlatformEnu, Ecef };

  /// Rotation frame
  RotationFrame rotationFrame = RotationFrame::SceneEnu;

  struct Entry {
    /// Date/time
    DateTime dateTime = {};

    /// Location
    geolib::Location location = {nullptr};

    /// Rotation angles
    std::array<double, 3> rotation = {0, 0, 0};
  };

  /// Entries
  std::vector<Entry> entries;

  void ReadXML(const xml::Node& node, const geolib::Facade* facade);

  void ReadPPDFile(const std::string& filename, const geolib::Facade* facade);
};

/// \}
}  // namespace d5::d4legacy
