/** Interface file for reading STK files
 *
 *  \file StkReportFile.h
 */

#pragma once

#include <vector>

#include "FlexMotion.h"
#include "dirsig5/d4legacy/Export.h"
#include "dirsig5/geolib/Location.h"

namespace d5::d4legacy::stk {

/// \addtogroup d4legacy_motion
/// \{

/** Loads ephemeris info from an STK report file
 *
 *  \param[in] filename  Filename from which to read
 *  \param[in] geolib    GeographicLib facade initialized with scene origin
 *
 *  \return List of ephemeris waypoints
 *  \throw std::exception subclass on any read error
 */
D5_D4LEGACY_EXPORT std::vector<LocationEngines::Waypoints::LocationEntry>
readEphemeris(const char* filename, geolib::Facade& geolib);

/// Result of reading the attitude data from an STK file
struct AttitudeResult {
  /// Time series of quaternion data
  std::vector<RotationEngines::Quaternions::QuaternionEntry> quaternions;

  /// Rotation matrix to apply to the quaternions (on the right)
  Eigen::Matrix3d bodyTransform;
};

/** Loads attitude info from an STK report file
 *
 *  \param[in] filename  Filename from which to read
 *
 *  \return The read attitude data
 *  \throw std::exception subclass on any read error
 */
D5_D4LEGACY_EXPORT AttitudeResult readAttitude(const char* filename);

/// \}
}  // namespace d5::d4legacy::stk
