#pragma once

#include "dirsig5/d4legacy/xml/xml.h"

namespace d5::d4legacy::platform {

struct D5_D4LEGACY_EXPORT Clock final {
public:
  /// The type, i.e., independent, dependent, external.
  std::string type{};
  /// The temporal units, e.g., hertz.
  std::string temporalUnits{"hertz"};
  /// The clock rate.
  double rate{-1.0};
  /// The clock offset.
  double offset{0.0};
  /// The clock divider.
  double divider{-1.0};
  /// The external filename if applicable.
  std::string externalFilename{};

public:
  /// Read XML. The given node is understood to be `<clock>`.
  void ReadXML(const xml::Node& node) {
    type = xml::requiredAttribute(node, "type").as_string();
    temporalUnits = node.attribute("temporalunits").as_string("hertz");
    rate = xml::readNumber(node, "rate", -1.0);
    offset = xml::readNumber(node, "offset", 0.0);
    divider = xml::readNumber(node, "divider", -1.0);
    externalFilename = node.child("filename").text().as_string();
  }
};

}  // namespace d5::d4legacy::platform
