#pragma once

#include "dirsig5/d4legacy/platform/Clock.h"
#include "dirsig5/d4legacy/xml/xml.h"

namespace d5::d4legacy::platform {

struct D5_D4LEGACY_EXPORT DetectorArray final {
public:
  /// The clock.
  Clock clock{};
  /// The element count in X.
  int elementCountX{0};
  /// The element count in Y.
  int elementCountY{0};
  /// The element size in X in microns.
  double elementSizeX{0.0};
  /// The element size in Y in microns.
  double elementSizeY{0.0};
  /// The element spacing in X in microns.
  double elementSpacingX{0.0};
  /// The element spacing in Y in microns.
  double elementSpacingY{0.0};
  /// The array offset in X in microns.
  double arrayOffsetX{0.0};
  /// The array offset in Y in microns.
  double arrayOffsetY{0.0};
  /// Flip axis in X?
  bool flipAxisX{false};
  /// Flip axis in Y?
  bool flipAxisY{false};
  /// The rolling readout delay.
  double rollingReadout{0.0};
  /// The external filename if applicable.
  std::string externalFilename{};
  /// The external file format if applicable.
  std::string externalFormat{};
  /// The field count if applicable.
  int fieldCount{};

public:
  /// Read XML. The given node is understood to be `<detectorarray>`.
  void ReadXML(const xml::Node& node) {
    clock.ReadXML(xml::requiredChild(node, "clock"));
    elementSizeX = xml::readNumber(node, "xelementsize", 0.0);
    elementSizeY = xml::readNumber(node, "yelementsize", 0.0);
    elementSpacingX = xml::readNumber(node, "xelementspacing", 0.0);
    elementSpacingY = xml::readNumber(node, "yelementspacing", 0.0);
    arrayOffsetX = xml::readNumber(node, "xarrayoffset", 0.0);
    arrayOffsetY = xml::readNumber(node, "yarrayoffset", 0.0);
    flipAxisX = node.child("xflipaxis").text().as_bool();
    flipAxisY = node.child("yflipaxis").text().as_bool();
    rollingReadout = node.attribute("rollingreadout").as_double();
    externalFilename = node.child("filename").text().as_string();
    externalFormat = node.child("format").text().as_string();
    fieldCount = node.child("fieldcount").text().as_int();
  }
};

}  // namespace d5::d4legacy::platform
