#pragma once

#include "dirsig5/d4legacy/xml/xml.h"

namespace d5::d4legacy::platform {

struct D5_D4LEGACY_EXPORT DetectorModel final {
public:
  /// The quantum efficiency.
  double quantumEfficiency{1.0};
  /// The read noise.
  double readNoise{0.0};
  /// The dark current density.
  double darkCurrentDensity{0.0};
  /// The minimum number of electrons.
  double minElectrons{0.0};
  /// The maximum number of electrons.
  double maxElectrons{1.0e20};
  /// The quantization bit depth.
  int bitDepth{0};

public:
  /// Read XML. The given node is understood to be `<detectormodel>`.
  void ReadXML(const xml::Node& node) {
    quantumEfficiency = xml::readNumber(node, "quantumefficiency", 1.0);
    readNoise = xml::readNumber(node, "readnoise", 0.0);
    darkCurrentDensity = xml::readNumber(node, "darkcurrentdensity", 0.0);
    minElectrons = xml::readNumber(node, "minelectrons", 0.0);
    maxElectrons = xml::readNumber(node, "maxelectrons", 1.0e20);
    bitDepth = int(xml::readNumber(node, "bitdepth", 0));
  }
};

}  // namespace d5::d4legacy::platform
