#pragma once

#include <optional>

#include "dirsig5/d4legacy/platform/DetectorArray.h"
#include "dirsig5/d4legacy/platform/DetectorModel.h"
#include "dirsig5/d4legacy/platform/GainResponse.h"
#include "dirsig5/d4legacy/platform/SpectralResponse.h"
#include "dirsig5/d4legacy/platform/TruthCollection.h"
#include "dirsig5/d4legacy/xml/xml.h"

namespace d5::d4legacy::platform {

struct D5_D4LEGACY_EXPORT FocalPlane final {
public:
  struct Extras {
    struct ActiveArea {
      /// The image filename.
      std::string imageFilename{};
    };
    struct StrayLight {
      /// The FRED grid filename.
      std::string gridFilename{};
      /// The fraction.
      double fraction{0.0};
    };
    struct Psf {
      /// The image filename.
      std::string imageFilename{};
      /// The FRED grid filename. Meant to be either-or with imageFilename.
      std::string gridFilename{};
      /// The parameter width, if using built-in Gaussian PSF.
      double parameterWidth{0.0};
      /// The scale.
      double scale{1.0};
    };
    struct Convergence {
      /// The convergence threshold.
      double threshold{0};
      /// The minimum number of samples.
      int minSamples{1};
      /// The maximum number of samples.
      int maxSamples{1};
      /// The maximum number of nodes.
      int maxNodes{5};
    };
    /// The active area parameters.
    std::optional<ActiveArea> activeArea{};
    /// The stray light parameters.
    std::optional<StrayLight> strayLight{};
    /// The point-spread-function parameters.
    std::optional<Psf> psf{};
    /// The default convergence parameters.
    std::optional<Convergence> convergence{};
  };

  bool enabled{true};
  /// The name.
  std::string name{};
  /// The name of the capture.
  std::string captureName{};
  /// The type of the capture (simple, raw, python, protolidar).
  std::string captureType{};
  /// The image file.
  ImageFile imageFile{};
  /// The detector array.
  std::optional<DetectorArray> detectorArray{};
  /// The detector model.
  std::optional<DetectorModel> detectorModel{};
  /// The spectral response.
  std::optional<SpectralResponse> spectralResponse{};
  /// The gain response.
  std::optional<GainResponse> gainResponse{};
  /// The spatial subsamples in X (for raw capture type).
  int spatialSubsamplesX{};
  /// The spatial subsamples in Y (for raw capture type).
  int spatialSubsamplesY{};
  /// The hypersampling multiplier.
  double hypersamplingMultiplier{1};
  /// The integration time if applicable.
  double integrationTime{0};
  /// Use TDI?
  bool useTdi{false};
  /// The truth collection parameters.
  std::optional<TruthCollection> truthCollection{};
  /// The output compression level (for lidar).
  std::string compressionLevel{};
  /// Extras, generally lesser-used.
  Extras extras{};

public:
  /// Read XML. The given node is understood to be `<focalplane>`.
  void ReadXML(const xml::Node& node);
};

}  // namespace d5::d4legacy::platform
