#pragma once

#include "dirsig5/d4legacy/xml/xml.h"

namespace d5::d4legacy::platform {

struct D5_D4LEGACY_EXPORT GainResponse final {
public:
  struct GainModel {
    /// The linear gain.
    double gain{1.0};
    /// The linear bias.
    double bias{0.0};
    /// The nonlinear LUT for input-output gain mapping.
    std::vector<std::pair<double, double>> nonlinearLut{};
  };
  std::vector<GainModel> models;

public:
  /// Read XML. The given node is understood to be `<gainresponse>`.
  void ReadXML(const xml::Node& node) {
    using namespace std::string_view_literals;
    models.clear();
    for (auto each : node.children("gainmodel")) {
      auto& model{models.emplace_back()};
      if (xml::requiredAttribute(each, "type").as_string() == "linear"sv) {
        model.gain = xml::readNumber(each, "gain", 1.0);
        model.bias = xml::readNumber(each, "bias", 0.0);
      } else {
        for (auto entry : xml::requiredChild(each, "lut").children("entry")) {
          model.nonlinearLut.emplace_back(
            xml::readNumber(entry, "input"),  //
            xml::readNumber(entry, "output"));
        }
      }
    }
  }
};

}  // namespace d5::d4legacy::platform
