#pragma once

#include "dirsig5/d4legacy/xml/xml.h"

namespace d5::d4legacy::platform {

struct D5_D4LEGACY_EXPORT ImageFile final {
public:
  /// The image file basename.
  std::string basename{};
  /// The image file extension.
  std::string extension{};
  /// The schedule (capture, task, simulation).
  std::string schedule{};
  /// The flux units (watts, milliwatts, microwatts, photonspersecond).
  std::string fluxunits{};
  /// The area units (m2, cm2).
  std::string areaunits{};
  /// The ENVI-style datatype.
  std::string datatype{};
  /// The default bands.
  std::vector<int> defaultBands{};

public:
  /// Read XML. The given node is understood to be `<imagefile>`.
  void ReadXML(const xml::Node& node) {
    basename = xml::requiredChild(node, "basename").text().as_string();
    extension = xml::requiredChild(node, "extension").text().as_string();
    schedule = xml::requiredChild(node, "schedule").text().as_string();
    fluxunits = node.child("fluxunits").text().as_string();
    areaunits = node.child("areaunits").text().as_string();
    datatype = node.child("datatype").text().as_string();
    defaultBands.clear();
    if (auto child = node.child("defaultbands"))
      for (auto index : TokenList(child.text().as_string(), "\r\n\t ,"))
        defaultBands.emplace_back(ParseNumber<int>(index));
  }

  /// Find and read XML subnodes beneath the given node because of stupid
  /// multi-conventions (`<imagebasename>`, `<imagefilename>`, or
  /// `<imagefile>`).
  void FindAndReadXMLSubnodes(const xml::Node& node) {
    *this = {};
    if (auto subnode = node.child("imagebasename")) {
      basename = subnode.text().as_string(), extension = "img",
      schedule = "collection";
    }
    if (auto subnode = node.child("imagefilename")) {
      basename = subnode.text().as_string(), extension = "img",
      schedule = "collection";
    }
    if (auto subnode = node.child("imagefile")) ReadXML(subnode);
  }
};

}  // namespace d5::d4legacy::platform
