#pragma once

#include <string>

#include "dirsig5/base/Geometry.h"
#include "dirsig5/d4legacy/xml/xml.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_plumes
/// \{

/// Holds the data necessary to specify a Blackadar plume
struct BlackadarData {
  /// The broad-band, incident radiative flux [W/m^2] (typically 300 - 600)
  double directInsolation = 0;

  /// The ambient air temperature [K] (typically 280 - 320)
  double airTemperature = 0;

  /// The wind speed at the base of the stack [m/s] (typically 1 - 5)
  double windSpeed = 0;

  /// The wind direction at the base of the stack [degrees, CW from N]
  double windDirection = 0;

  /// The surface roughness length [m] (typically (0.01 - 0.10)
  double surfaceRoughness = 0;

  /// The altitude of the mixed layer [m] (typically 500 - 2000)
  double mixedAltitude = 0;

  /// The X/Y/Z location of the stack base [m]
  Point stackLocation;

  /// The height of the stack [m] (typically 60 - 150)
  double stackHeight = 0;

  /// The diameter of the stack [m] (typically 1 - 4)
  double stackDiameter = 0;

  /// The Briggs buoyancy flux parameter (F) at the stack [m^4/s^3] (typically
  /// 4 - 8)
  double releaseBuoyancy = 0;

  /// The exiting stack temperature [K] (typically 300 - 400)
  double releaseTemperature = 0;

  /// The number of time steps to establish the plume
  size_t stepCount = 0;

  /// The number of new puffs released each time step
  size_t puffsPerStep = 0;

  /// The ID of the exiting material
  std::string plumeMaterialId;

  /// The release rate of the exiting material [ppm/s] (typically 100 -
  /// 10,000)
  double releaseRate = 0;

  /// The (optional) name of the file that debug info is written to
  std::string debugFilename;

  /// Currently unused as some equations assume a 1 second timestep
  double stepTime = -1;

  void ReadXML(const xml::Node& node);
};

/// \}
}  // namespace d5::d4legacy
