/** Interface file for SceneFile
 *
 *  \file SceneFile.h
 */

#pragma once

#include <array>
#include <locale>
#include <memory>
#include <optional>
#include <string>
#include <unordered_map>

#include "dirsig5/d4legacy/Export.h"
#include "dirsig5/d4legacy/misc/FileLocator.h"
#include "dirsig5/d4legacy/plumes/Blackadar.h"
#include "dirsig5/d4legacy/scene/Maps.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_scene
/// \{

struct D5_D4LEGACY_EXPORT GeometryListInclude {
  std::string name;
  bool enabled = true;
  std::string filename;
};

class D5_D4LEGACY_EXPORT SceneFile {
public:
  SceneFile() = default;
  SceneFile(const SceneFile&) = delete;
  SceneFile& operator=(const SceneFile&) = delete;
  SceneFile(SceneFile&&) = default;
  SceneFile& operator=(SceneFile&&) = default;

  static std::unique_ptr<SceneFile> LoadFromXmlFile(
    const std::string& filename);

  struct ThumbnailInfo {
    std::string name;
    std::string description;
    std::string filename;
  };

public:
  std::unordered_map<std::string, std::string> metadata;
  std::string sceneName;
  std::string sceneDir;
  FileLocator fileLocator;
  std::array<double, 3> latLonAlt;
  double groundAltitude = 0;
  std::string materialFilename;
  std::vector<GeometryListInclude> glistIncludes;
  std::vector<ThumbnailInfo> thumbnailInfo;
  std::vector<std::string> attributes;
  double spectralDelta = 0;
  double minGsd = 0;
  double maxGsd = 0;
  std::vector<std::unique_ptr<MapEntry>> mapEntries;
  std::vector<BlackadarData> blackadarPlumes;
};

/// \}
}  // namespace d5::d4legacy
