#pragma once

#include <array>
#include <string>
#include <utility>
#include <variant>
#include <vector>

#include "dirsig5/d4legacy/mat/SimpleSpectralData.h"

namespace d5::d4legacy {

/// \addtogroup d4legacy_sources
/// \{

/// Information about sources
struct SourceInfo {
  /// Intensity spectrum (.int) filename
  std::string intensityFilename;

  /// Shape factor IES filename, overrides shapeFactor/shapeFactorMagnitudes
  /// if present.
  std::string shapeFactorIESFilename;

  /// Shape factor (cosine exponent) for shaping
  double shapeFactor = 0.0;

  /// Vector of shape factor/magnitudes for shaping
  std::vector<std::pair<double, double>> shapeFactorMagnitudes;

  /// Whether to normalizae the shape factors
  bool normalizeShape = true;

  /// Filename for modulation inputs
  std::string psdFilename;

  /// Frequency in Hz of blinking
  double blinkFrequency = 0;

  /// Offset of blinking in seconds
  double blinkOffset = 0;

  /// Whether the source is photon mapped
  bool photonMapped = false;

  /// Original ID of the source
  std::string id;

  /// Original name of the source
  std::string name;
};

D5_D4LEGACY_EXPORT SimpleSpectralData
ReadSourceIntensityFile(const std::string& filename);

D5_D4LEGACY_EXPORT std::vector<std::array<double, 3>> ReadPsdFile(
  const std::string& filename);

/// \}
}  // namespace d5::d4legacy
