/** Interface file for TagFile
 *
 *  \file TagFile.h
 */

#pragma once

#include <cstdint>
#include <istream>
#include <string>
#include <vector>

#include "dirsig5/d4legacy/Export.h"
#include "dirsig5/d4legacy/tag/TagFileElement.h"

namespace d5::d4legacy {

/**
 * \class TagFile
 * \ingroup d4legacy_tag
 *
 * Reader for DIRSIG3 style tag file.
 */
class D5_D4LEGACY_EXPORT TagFile {
public:
  /// The destructor
  ~TagFile() = default;

  /// Gets the top-most section of the file
  [[nodiscard]] TagFileSection& topSection() { return topSection_; }

  /// Gets the top-most section of the file
  [[nodiscard]] const TagFileSection& topSection() const { return topSection_; }

  /** Loads a tag file from an input stream
   *
   *  \param is  The input stream from which to read
   *
   *  \return A non-null pointer to a tag file on success. Nullptr is never
   *          returned.
   *  \throws std::exception subclass if an error is encountered
   */
  static std::unique_ptr<TagFile> LoadFromIstream(std::istream& is);

private:
  TagFileSection topSection_;
};
}  // namespace d5::d4legacy
