/** Interface file for wavelength units
 *
 *  \file WavelengthUnits.h
 */

#pragma once

#include <cstdint>
#include <string>
#include <string_view>

#include "dirsig5/envi/Export.h"

namespace d5::envi {

/// Measurement units of wavelength
enum class WavelengthUnits : std::uint8_t {
  Micrometers = 0,
  Nanometers,
  Millimeters,
  Centimeters,
  Meters,
  Angstroms,
  Wavenumbers,
  GHz,
  MHz,
  Index,
  Unknown
};

/** Converts a units constant to a string
 *
 *  \param val  The constant to convert
 *
 *  \return A string representation of the units
 *  \throw std::domain_error on an invalid input
 */
D5_ENVI_EXPORT const std::string &to_string(WavelengthUnits val);
} // namespace d5::envi
