#pragma once

#include "dirsig5/flex_motion/LocationEngine.h"

namespace d5::flex_motion::LocationEngines {

/// An object moving in a circle at a constant altitude and speed.
class D5_FM_EXPORT Circling final : public LocationEngine {
public:
  Circling() = default;

  /// The location transform is constant for this engine
  LocationState doGet(const DateTime &, double relativeTime) const override {
    double lapAngle = speed / radius * relativeTime;
    double sinLapAngle = std::sin(lapAngle);
    double cosLapAngle = std::cos(lapAngle);
    Point location = {radius * cosLapAngle, radius * sinLapAngle, 0.0};
    Vector velocity = {speed * sinLapAngle, speed * cosLapAngle, 0.0};
    return {location + center, velocity};
  }

public:
  /// The center location
  Point center = {0, 0, 0};

  /// The fixed radius from the center [m]
  double radius = 0;

  /// The fixed speed [m/s]
  double speed = 0;
};

} // namespace d5::flex_motion::LocationEngines
