#pragma once

#include "dirsig5/flex_motion/LocationEngine.h"

namespace d5::flex_motion::LocationEngines {

/// An object moving as defined by an orbit predicted by the SGP4 model.
class D5_FM_EXPORT Sgp4 final : public LocationEngine {
public:
  Sgp4();
  Sgp4(const Sgp4 &) = delete;
  Sgp4(Sgp4 &&) = default;
  Sgp4 &operator=(const Sgp4 &) = delete;
  Sgp4 &operator=(Sgp4 &&) noexcept = delete;

  ~Sgp4() override;

  void doUpdateState(const StateDateTime &, const geolib::Facade *) override;

  [[nodiscard]] LocationState doGet(const DateTime &, double) const override;

public:
  /// The ECEF to ENU transform
  Matrix ecefToEnuTransform = Matrix::Identity();

  /// The GeographicLib wrapper
  const geolib::Facade *geoLibFacade = nullptr;

  /// The first line of the Two-Line Element (TLE)
  char tleLine1[70] = {};

  /// The second line of the Two-Line Element (TLE)
  char tleLine2[70] = {};

  /// Internal implementation class (PIMPL idiom)
  struct Impl;

  /// The record parsed from the Two-Line Element (TLE)
  mutable Impl *pimpl_;
};

} // namespace d5::flex_motion::LocationEngines
