#pragma once

#include "dirsig5/flex_motion/LocationEngine.h"

namespace d5::flex_motion::LocationEngines {

/// An object moving at a constant altitude, speed, and heading.
class D5_FM_EXPORT StraightPath final : public LocationEngine {
public:
  StraightPath() = default;

  LocationState doGet(const DateTime &, double relativeTime) const override {
    Point velocity(speed * std::sin(toRadians(heading)),
                   speed * std::cos(toRadians(heading)), 0.0);
    return {startLocation + relativeTime * velocity, velocity};
  }

public:
  /// The fixed (compass) heading [degrees]
  double heading = 0;

  /// The fixed speed [m/s]
  double speed = 0;

  /// The start location
  Point startLocation = {0, 0, 0};
};

} // namespace d5::flex_motion::LocationEngines
