#pragma once

#include "dirsig5/flex_motion/LocationEngine.h"

namespace d5::flex_motion::LocationEngines {

/// This location models an object moving along a set of waypoints.
class D5_FM_EXPORT Waypoints final : public LocationEngine {
public:
  Waypoints() = default;

  struct Keyframe {
    double timeSec = 0;
    Point location = {0, 0, 0};
  };

  [[nodiscard]] bool isStatic() const override { return keyframes.size() <= 1; }

  void doUpdateState(const StateDateTime &, const geolib::Facade *) override {
    std::sort(keyframes.begin(), keyframes.end(),
              [&](const Keyframe &lhs, const Keyframe &rhs) {
                return lhs.timeSec < rhs.timeSec;
              });
  }

  [[nodiscard]] LocationState doGet(const DateTime &, double) const override;

public:
  /// The list of known waypoints
  std::vector<Keyframe> keyframes;

  /// The flag that defines if the supplied data should be looped
  bool loop = false;
};

} // namespace d5::flex_motion::LocationEngines
