#pragma once

#include <optional>

#include "dirsig5/flex_motion/Motion.h"

namespace d5::flex_motion::Motions {

/// Implements a data-driven model, by storing the motion as a series of
/// known motion states as a function of time.
class D5_FM_EXPORT DataDriven final : public Motion {
public:
  DataDriven() = default;

  struct Keyframe {
    double timeSec = 0;
    DualQuaternion transform = {};
  };

  void doUpdateState(const StateDateTime &, const geolib::Facade *) override;

  [[nodiscard]] DualQuaternion doGet(const DateTime &) const override;

public:
  /// The list of known motion states (locations and orientations)
  std::vector<Keyframe> keyframes;

  bool loop = false;

  DateTime startDateTime = DateTime::fromRelativeSeconds(0);
};

} // namespace d5::flex_motion::Motions
