#pragma once

#include "dirsig5/base/Clock.h"
#include "dirsig5/flex_motion/Motion.h"

namespace d5::flex_motion {

class D5_FM_EXPORT Mount : public Motion {
protected:
  DualQuaternion doGet(const DateTime &dateTime) const final override;

public:
  Clock clock = {};

  DateTime startDateTime = DateTime::fromRelativeSeconds(0);

protected:
  virtual DualQuaternion doGet(const DateTime &absoluteTime,
                               double relativeTime,
                               double scanFraction) const = 0;
};

inline DualQuaternion Mount::doGet(const DateTime &dateTime) const {
  auto absoluteTime = cache.referenceDateTime.makeAbsolute(dateTime);
  auto startTime = cache.referenceDateTime.makeAbsolute(startDateTime);
  double relativeTime = (absoluteTime - startTime).getTotalSeconds();
  double scanFraction =
      clock.clockRate * std::fmod(relativeTime, 1.0 / clock.clockRate);
  return doGet(absoluteTime, relativeTime, scanFraction);
}

} // namespace d5::flex_motion
