#pragma once

#include <array>

#include "dirsig5/flex_motion/Mount.h"

namespace d5::flex_motion::Mounts {

/// A data-driven mount model.
class D5_FM_EXPORT DataDriven final : public Mount {
public:
  DataDriven() = default;

  struct Keyframe {
    double timeUnits = 0;
    std::array<double, 3> rotation = {0, 0, 0};
    std::array<double, 3> translation = {0, 0, 0};
  };

public:
  std::vector<Keyframe> keyframes = {};

  bool loop = true;

  bool useScanFraction = true;

  std::array<char, 3> rotationOrder = {'x', 'y', 'z'};

protected:
  void doUpdateState(const StateDateTime &, const geolib::Facade *) override {
    std::sort(keyframes.begin(), keyframes.end(),
              [](const Keyframe &lhs, const Keyframe &rhs) {
                return lhs.timeUnits < rhs.timeUnits;
              });
  }

  DualQuaternion doGet(const DateTime &, double, double) const override;
};

} // namespace d5::flex_motion::Mounts
