#pragma once

#include "dirsig5/flex_motion/RotationEngine.h"

namespace d5::flex_motion::RotationEngines {

/// Rotation defined by triplets of Euler angles with a given rotation order.
class D5_FM_EXPORT EulerAngles final : public RotationEngine {
public:
  EulerAngles() = default;

  struct Keyframe {
    double timeSec = 0;

    std::array<double, 3> angles = {0, 0, 0};
  };

  bool isStatic() const override { return keyframes.size() <= 1; }

  void doUpdateState(const StateDateTime &, const geolib::Facade *) override {
    std::sort(keyframes.begin(), keyframes.end(),
              [](const Keyframe &lhs, const Keyframe &rhs) {
                return lhs.timeSec < rhs.timeSec;
              });
  }

  Quaternion doGet(const DateTime &, double) const override;

public:
  std::vector<Keyframe> keyframes;

  bool loop = false;

  std::array<char, 3> rotationOrder = {'x', 'y', 'z'};
};

} // namespace d5::flex_motion::RotationEngines
