#pragma once

#include "dirsig5/flex_motion/RotationEngine.h"

namespace d5::flex_motion::RotationEngines {

/// This engine models an object with the orientation such that it is
/// always pointed at the current locations computed by a location engine.
/// That location engine could be modeling a fixed location, which makes
/// this a spotlight situation.  Or that location engine could be modeling
/// a dynamic location, which makes this a tracking situation.  Since the
/// line-of-sight between the object location and the location it is
/// tracking does not fully define the orientation, we have some constraint
/// options that include the platform velocity and the "up" direction.
class D5_FM_EXPORT LookAt final : public RotationEngine {
public:
  LookAt() = default;

  bool isStatic() const override {
    return (locationEngine and locationEngine->isStatic()) and //
           (trackingEngine and trackingEngine->isStatic());
  }

  void doUpdateState(const StateDateTime &state,
                     const geolib::Facade *facade) override {
    if (trackingEngine)
      trackingEngine->updateState(state, facade);
  }

  Quaternion doGet(const DateTime &, double) const override;

public:
  std::shared_ptr<LocationEngine> trackingEngine = {};
};

} // namespace d5::flex_motion::RotationEngines
