#pragma once

#include "dirsig5/flex_motion/RotationEngine.h"

namespace d5::flex_motion::RotationEngines {

/// Rotation defined by quaternions.
class D5_FM_EXPORT Quaternions final : public RotationEngine {
public:
  Quaternions() = default;

  struct Keyframe {
    double timeSec = 0;
    Quaternion quaternion = {1, 0, 0, 0};
  };

  bool isStatic() const override { return keyframes.size() <= 1; }

  void doUpdateState(const StateDateTime &, const geolib::Facade *) override {
    std::sort(keyframes.begin(), keyframes.end(),
              [](const Keyframe &lhs, const Keyframe &rhs) {
                return lhs.timeSec < rhs.timeSec;
              });
  }

  Quaternion doGet(const DateTime &, double) const override;

public:
  std::vector<Keyframe> keyframes;

  bool loop = false;

  Quaternion localTransform = {1, 0, 0, 0};
};

} // namespace d5::flex_motion::RotationEngines
