#pragma once

#include "dirsig5/flex_motion/RotationEngine.h"

namespace d5::flex_motion::RotationEngines {

/// An object with orientation defined by a spin rate and axis.
class D5_FM_EXPORT Spin final : public RotationEngine {
public:
  Spin() = default;

  /// A type to store the rotation rate for a given time.
  struct Keyframe {
    double timeSec = 0; /// The time [seconds]
    double rate = 0;    /// The rotation rate [radians/second]

    struct Cache {
      double angle = 0; /// The integrated angle
    } cache = {};
  };

  void doUpdateState(const StateDateTime &, const geolib::Facade *) override;

  Quaternion doGet(const DateTime &, double) const override;

public:
  /// The vector of rate entries vs. time
  std::vector<Keyframe> keyframes;

  /// The transform to go from the rotation axis to DIRSIG
  Vector spinAxis = {0, 0, 1};
};

} // namespace d5::flex_motion::RotationEngines
