/** Interface for the MPI driver abstraction
 *
 *  \file Driver.h
 */

#pragma once

#include "dirsig5/mpi/Types.h"

namespace d5::mpi {

class Driver {
public:
  virtual ~Driver() = default;

  virtual int Barrier(Comm comm) = 0;

  virtual Comm COMM_SELF() = 0;

  virtual Comm COMM_WORLD() = 0;

  virtual int Comm_rank(Comm comm, int *rank) = 0;

  virtual int Comm_size(Comm comm, int *size) = 0;

  virtual int Error_string(int errorcode, char *string, int *resultlen) = 0;

  virtual int Get_count(const Status *status, DataType datatype,
                        int *count) = 0;

  virtual int Get_processor_name(char *name, int *resultlen) = 0;

  virtual File FILE_NULL() = 0;

  virtual int File_close(File *fh) = 0;

  virtual int File_delete(const char *filename, Info info) = 0;

  virtual int File_get_position(File fh, mpi::Offset *offset) = 0;

  virtual int File_open(Comm comm, const char *filename, int amode, Info info,
                        File *file) = 0;

  virtual int File_seek(File fh, mpi::Offset offset, int whence) = 0;

  virtual int File_sync(File fh) = 0;

  virtual int File_write(File fh, const void *buf, int count, DataType datatype,
                         Status *status) = 0;

  virtual int File_write_at(File fh, mpi::Offset offset, const void *buf,
                            int count, DataType datatype, Status *status) = 0;

  virtual int Finalize() = 0;

  virtual Info INFO_NULL() = 0;

  virtual int Initialized(int *flag) = 0;

  virtual int Init_thread(int *argc, char ***argv, Thread required,
                          Thread *provided) = 0;

  virtual int MAX_ERROR_STRING() = 0;

  virtual int MAX_PROCESSOR_NAME() = 0;

  virtual int Probe(int source, int tag, Comm comm, Status *status) = 0;

  virtual int Recv(void *buf, int count, DataType datatype, int source, int tag,
                   Comm comm, Status *status) = 0;

  virtual int Send(const void *buf, int count, DataType datatype, int dest,
                   int tag, Comm comm) = 0;
};

} // namespace d5::mpi
