/** Interface file for the DIRSIG5 interface to MPI
 *
 *  \file Mpi.h
 */

#pragma once

#include <optional>
#include <string>

#include "dirsig5/mpi/Export.h"
#include "dirsig5/mpi/Types.h"

namespace d5::mpi {

class Driver;

class D5_MPI_EXPORT Interface {
public:
  static Interface &getInstance();

  void Barrier(Comm comm);

  Comm COMM_SELF();

  Comm COMM_WORLD();

  int Comm_rank(Comm comm);

  int Comm_size(Comm comm);

  std::string Error_string(int errorcode);

  File FILE_NULL();

  void File_close(File *fh);

  void File_delete(const char *filename, Info info);

  mpi::Offset File_get_position(File fh);

  File File_open(Comm comm, const char *filename, int amode, Info info);

  std::optional<File> File_open_try(Comm comm, const char *filename, int amode,
                                    Info info);

  void File_seek(File fh, mpi::Offset offset, int whence);

  void File_sync(File fh);

  Status File_write(File fh, const void *buf, int count, DataType datatype);

  Status File_write_at(File fh, mpi::Offset offset, const void *buf, int count,
                       DataType datatype);

  void Finalize();

  int Get_count(const Status *status, DataType datatype);

  std::string Get_processor_name();

  Info INFO_NULL();

  bool Initialized();

  Thread Init_thread(int *argc, char ***argv, Thread required);

  Status Probe(int source, int tag, Comm comm);

  Status Recv(void *buf, int count, DataType datatype, int source, int tag,
              Comm comm);

  void Send(const void *buf, int count, DataType datatype, int dest, int tag,
            Comm comm);

private:
  Interface();
  ~Interface();

  void CheckSuccess(int errorcode);

private:
  Driver *driver_;
};
} // namespace d5::mpi

namespace d5 {

inline mpi::Interface &MPI() { return mpi::Interface::getInstance(); }

} // namespace d5
