/** Interface file for MPI types
 *
 *  \file Types
 */

#pragma once

#include <cstdint>
#include <optional>
#include <string>

namespace d5::mpi {

#define MPI_DATA_TYPES(X)                                                      \
  X(CHAR, char)                                                                \
  X(SIGNED_CHAR, signed char)                                                  \
  X(UNSIGNED_CHAR, unsigned char)                                              \
  X(BYTE, unsigned char)                                                       \
  X(WCHAR, wchar_t)                                                            \
  X(SHORT, short)                                                              \
  X(UNSIGNED_SHORT, unsigned short)                                            \
  X(INT, int)                                                                  \
  X(UNSIGNED, unsigned int)                                                    \
  X(LONG, long)                                                                \
  X(UNSIGNED_LONG, unsigned long)                                              \
  X(LONG_LONG_INT, long long)                                                  \
  X(LONG_LONG, long long)                                                      \
  X(UNSIGNED_LONG_LONG, unsigned long long)                                    \
  X(FLOAT, float)                                                              \
  X(DOUBLE, double)                                                            \
  X(LONG_DOUBLE, long double)                                                  \
  X(INT8_T, int8_t)                                                            \
  X(INT16_T, int16_t)                                                          \
  X(INT32_T, int32_t)                                                          \
  X(INT64_T, int64_t)                                                          \
  X(UINT8_T, uint8_t)                                                          \
  X(UINT16_T, uint16_t)                                                        \
  X(UINT32_T, uint32_t)                                                        \
  X(UINT64_T, uint64_t)                                                        \
  X(C_BOOL, _Bool)                                                             \
  X(C_FLOAT_COMPLEX, float _Complex)                                           \
  X(C_COMPLEX, float _Complex)                                                 \
  X(C_DOUBLE_COMPLEX, double _Complex)                                         \
  X(C_LONG_DOUBLE_COMPLEX, long double _Complex)

#define X_ENUM_DECL(name, t) name, /* NOLINT */

enum DataType { DATATYPE_NULL = 0, MPI_DATA_TYPES(X_ENUM_DECL) };

#undef X_ENUM_DECL

#define MPI_FILE_MODES(X)                                                      \
  X(MODE_APPEND, 0x1)                                                          \
  X(MODE_CREATE, 0x2)                                                          \
  X(MODE_DELETE_ON_CLOSE, 0x4)                                                 \
  X(MODE_EXCL, 0x8)                                                            \
  X(MODE_RDONLY, 0x10)                                                         \
  X(MODE_RDWR, 0x20)                                                           \
  X(MODE_SEQUENTIAL, 0x40)                                                     \
  X(MODE_WRONLY, 0x80)                                                         \
  X(MODE_UNIQUE_OPEN, 0x100)

#define X_ENUM_DECL(name, val) name = val, /* NOLINT */

enum FileMode { MPI_FILE_MODES(X_ENUM_DECL) };

#undef X_ENUM_DECL

using Offset = std::int64_t;
using Comm = struct comm_t *;
using File = struct file_t *;
using Info = struct info_t *;

#define MPI_THREAD_TYPES(X)                                                    \
  X(THREAD_SINGLE, 0)                                                          \
  X(THREAD_FUNNELED, 1)                                                        \
  X(THREAD_SERIALIZED, 2)                                                      \
  X(THREAD_MULTIPLE, 3)

#define X_ENUM_DECL(name, val) name = val, /* NOLINT */

enum Thread : int { MPI_THREAD_TYPES(X_ENUM_DECL) };

#undef X_ENUM_DECL

// 64 bytes should cover most MPI implementations
static constexpr int MaxStatusSize = 64;

struct Status {
  int MPI_SOURCE;
  int MPI_TAG;
  int MPI_ERROR;
  char original_[MaxStatusSize];
};
} // namespace d5::mpi
