#pragma once

#include "dirsig5/apis/RasterInputInterface.h"
#include "dirsig5/raster/Raster.h"

/** \defgroup raster Raster Input Library
 *  Holds routines for reading raster data
 */

/// Main namespace for the raster input library
namespace d5::raster {

/// \addtogroup raster
/// \{

/** Loads a raster image
 *
 *  \param request  The read request object
 *
 *  \return The read raster data
 *  \throw std::exception subclass on any error
 */
D5_RASTER_EXPORT Raster ReadRaster(const apis::RasterInputRead::In &request);

/** Trys to load metadata from an image
 *
 *  \param[in] request  The read request object
 *  \param[out] result  The result struct to populate
 *
 *  \return Whether any metadata could be read
 */
D5_RASTER_EXPORT bool
TryToReadRasterMetadata(const apis::RasterMetadataRead::In &request,
                        apis::RasterMetadataRead::Out &result);

/// \}
} // namespace d5::raster
