#pragma once

#include <vector>

#include "dirsig5/base/Constants.h"
#include "dirsig5/base/Geometry.h"
#include "dirsig5/base/Temperature.h"
#include "dirsig5/thermal/Export.h"
#include "dirsig5/thermal/RitThermInvariants.h"

namespace d5::thermal {

/// A record in the THERM table
struct ThermRecord {
  /// Specific heat in [cal/g/C]
  float specificHeat;

  /// Mass density in [g/cm^3]
  float massDensity;

  /// Thermal conductivity in [cm*Langley)/hour/C]
  float thermalConductivity;

  /// Thickness of the material in [cm]
  float thickness;

  /// The solar absorption of the material
  float solarAbsorption;

  /// Thermal emissivity of the material
  float thermalEmissivity;

  /// Self-generated power of the material [Langley/hour]
  float selfGeneratedPower;

  /// Exposed area term
  float exposedArea;
};

D5_THERMAL_EXPORT Temperature therm(
  const Vector& surfaceNormal,
  const ThermRecord& thermalProps,
  const std::vector<float>& sunShadowHistory,
  float diffuseInsolationWeight,
  const RitThermInvariants& thermInvariants,
  size_t startIndex);

}  // namespace d5::thermal
