#pragma once

#include <type_traits>
#include <vector>

#include "dirsig5/base/Geometry.h"
#include "dirsig5/base/span.h"

namespace d5::thermal {

/// A single data point in a shadow history (the time is not included)
struct ShadowHistorySample {
  /// [0,1] quantity (0 = no shadow, 1 = full shadow)
  float shadowFraction = 0;

  /// Unit vector towards the sun in the current coordinate system
  Vectorf solarVector = {};
};

/// Type trait to determine if something is a shadow history callback
template <typename Callable>
using is_shadow_history_callback = std::
  is_invocable_r<std::vector<ShadowHistorySample>, Callable, span<const float>>;

/// Convenience variable for is_shadow_history_callback::value
template <typename Callable>
inline constexpr bool is_shadow_history_callback_v =
  is_shadow_history_callback<Callable>::value;

/// Helper function to static_assert is_shadow_history_callback_v
template <typename Callable>
constexpr void assert_shadow_history_callback(Callable&&) {
  static_assert(
    is_shadow_history_callback_v<Callable>,
    "Callback parameter must be a shadow history callback: "
    "std::vector<ShadowHistorySample>(d5hdf::span<const float>)");
}
}  // namespace d5::thermal
