/** Implementation file for Common.h
 *
 *  \file Common.cpp
 */

#include "Common.h"

#include <cmath>

namespace d5::atm {

std::vector<double>
BlackbodyIrradiance(double temperature,
                    const std::vector<double> &wavelengths) {
  static const double C1m = 3.741832E-20; // [Watt cm^-2 m^1]
  static const double C2m = 1.438786E-02; // [m^-1 K]

  // loop through each spectral point in the vector
  std::vector<double> result(wavelengths.size());
  for (size_t ii = 0; ii < wavelengths.size(); ii++) {
    // get the spectral location of this index (in meters)
    double lm = wavelengths[ii] * 1.0e-06;

    // check if the temperature is set
    double value;
    if (temperature <= 0.0) {
      value = 0.0;
    } else {
      // manually compute lm^5
      double lm5 = lm * lm * lm * lm * lm;

      // compute the spectral value at this location
      value = C1m / (lm5 * (std::exp(C2m / (lm * temperature)) - 1.0));

      // convert the value from W/(cm^2 m) to W/(cm^2 um)
      value *= 1.0e-06;
    }

    result[ii] = value;
  }

  return result;
}
} // namespace d5::atm
