/** Common header file for backend utilities
 *
 *  \file Common.h
 */

#include <vector>

namespace d5::atm {

inline constexpr double PI = 3.14159265358979323846264338327950288;

/** Generates a Planck blackbody spectral irradiance curve for the supplied
 *  temperature and wavelengths.
 *
 *  \param temperature  The temperature in Kelvin
 *  \param wavelengths  The wavelengths in microns
 *
 *  \return The spectral irradicance in W/(cm^2 um)
 */
std::vector<double> BlackbodyIrradiance(
    double temperature, const std::vector<double>& wavelengths);
}
