/** Interface file for the dummy backend
 *
 *  \file DummyBackend.h
 */

#pragma once

#include "dirsig5/atmosphere/Atmosphere.h"

namespace d5::atm {

/// This class implements a simple "dummy" backend that is useful for testing
class DummyBackend : public ModelBackend {
public:
  /// \override
  [[nodiscard]] const std::string& name() const override {
    static const std::string kName = "DummyBackend";
    return kName;
  }

  /// \override
  void printSummary(std::ostream& stream) const override {
    stream << "Dummy backend";
  }

  /// \override
  [[nodiscard]] ModelResponse performRequest(
      const ModelRequest& request) const override;

private:
  /// Computes the exo-atmospheric solar irradiance.
  void computeExoSolar(
      double minLambda,
      double deltaLambda,
      std::size_t lambdaCount,
      ModelResponse& response) const;

  /// Computes the exo-atmospheric lunar irradiance.
  void computeExoLunar(
      double minLambda,
      double deltaLambda,
      std::size_t lambdaCount,
      ModelResponse& response) const;

  /// Computes the radiance and transmission along a path to the sky.
  void computeSkyPath(
      double minLambda,
      double deltaLambda,
      std::size_t lambdaCount,
      ModelResponse& response) const;

  /// Computes the radiance and transmission along non-sky paths.
  void computeOtherPath(
      double minLambda,
      double deltaLambda,
      std::size_t lambdaCount,
      ModelResponse& response) const;
};
}
