#pragma once

#include <cassert>
#include <cstring>
#include <istream>
#include <vector>

#include "dirsig5/atmosphere/backends/ModtranCommon.h"

namespace LegacyOutput {
/**
 * Read the 11-line header (a simplified "tape5") from the tape7 file.
 */
void readHeader(
    std::istream& tapeStream,
    float& h1,
    float& h2,
    float& angle,
    float& range,
    float& v1,
    float& v2,
    float& dv);

/**
 * Read the remaining data lines in the file from an irradiance run.
 */
void readIrradianceData(
    std::istream& tapeStream,
    std::vector<float>& totalIrrad,
    std::vector<float>& totalTau);

/**
 * Read the remaining data lines in the file from an radiance run
 * and scoop out the path radiance and transmission.
 */
void readPathRadianceData(
    std::istream& tapeStream,
    std::vector<float>& pathRad,
    std::vector<float>& pathTau);

/**
 * Read the remaining data lines in the file from an radiance run
 * and scoop out the total radiance and transmission.
 */
void readTotalRadianceData(
    std::istream& tapeStream,
    std::vector<float>& totalRad,
    std::vector<float>& totalTau);

/**
 * Read the remaining data lines in the file from an radiance run
 * and scoop out the sensor-reaching and surface-leaving radiance.
 */
void readSensorAndSurfaceData(
    std::istream& tapeStream,
    std::vector<float>& totalRad,
    std::vector<float>& surfaceRad);
}
