#ifndef NOMINMAX
#define NOMINMAX 1
#endif
#if defined(__linux__) || defined(__linux) || defined(linux) || defined(_LINUX)
#define HAS_LINUX 1
#elif defined(__APPLE__)
#define HAS_APPLE 1
#elif defined(_WIN32) || defined(_WIN64)
#define HAS_WINDOWS 1
#else
#error "Unsupported OS!"
#endif

#include "dirsig5/base/Filesystem.h"

#include <stdexcept>
#include <string>
#include <string_view>
using namespace std::string_literals;
using namespace std::string_view_literals;

#if HAS_LINUX || HAS_APPLE
#define HAS_UNIX 1
#define HOME_VAR "HOME"
#define LINE_END "\n"s
#else
#define HOME_VAR "USERPROFILE"
#define LINE_END "\r\n"s
#endif  // #if HAS_LINUX || HAS_APPLE

#if HAS_WINDOWS
#include <Windows.h>
#include <strsafe.h>

static std::string WindowsErrorString() {
  static_assert(sizeof(CHAR) == 1);
  LPVOID lpMsgBuf;
  DWORD dw = ::GetLastError();
  ::FormatMessageA(
      FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |
          FORMAT_MESSAGE_IGNORE_INSERTS,
      NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR)&lpMsgBuf, 0,
      NULL);

  LPVOID lpDisplayBuf =
      (LPVOID)::LocalAlloc(LMEM_ZEROINIT, (::lstrlenA((LPCSTR)lpMsgBuf) + 40));

  ::StringCchPrintfA(
      (LPSTR)lpDisplayBuf, ::LocalSize(lpDisplayBuf),
      "Failed with error %d: %s", dw, lpMsgBuf);

  std::string error = reinterpret_cast<const char*>(lpDisplayBuf);

  ::LocalFree(lpMsgBuf);
  ::LocalFree(lpDisplayBuf);

  return error;
}
#endif  // #if HAS_WINDOWS
