#include "dirsig5/base/LibraryManager.h"

#include "Config.h"
#include "dirsig5/base/Logger.h"
#include "dirsig5/base/StringUtils.h"

namespace d5 {

long LibraryManager::loadLibraryDirectory(const std::string &dirname) {
  long count = 0;
  if (stdfs::exists(dirname))
    for (const auto &direntry : stdfs::directory_iterator{dirname}) {
      if (stdfs::is_directory(direntry.path()))
        continue;
      std::string extension = toLower(direntry.path().extension().string());
      if (extension == ".so" or    //
          extension == ".dylib" or //
          extension == ".dll")
        count += loadLibrary(direntry.path().string());
    }
  return count;
}

bool LibraryManager::loadLibrary(const std::string &filename) {
  if (libraries.find(filename) != libraries.end())
    return false;
  if (Library library(filename); library) {
    if (void *func = library.resolve(funcName); func) {
      libraries[filename] = {library, func};
      return true;
    }
  }
  return false;
}

void *LibraryManager::getLibraryFunc(const std::string &name) const {
  // Look for a fully specified filename first.
  if (auto itr = libraries.find(name); itr != libraries.end())
    return itr->second.second;

  // Look for anything matching the basename, excluding "lib" on UNIX systems.
  for (const auto &[key, value] : libraries) {
    std::string stem = stdfs::path(key).stem().string();
    std::string_view stemView = stem;
    if (stemView.substr(0, 3) == "lib")
      stemView.remove_prefix(3);
    if (stemView == name)
      return value.second;
  }

  // Not found.
  return nullptr;
}

std::vector<std::string>
LibraryManager::FindFilenames(const std::string &dirname) {
  std::vector<std::string> filenames;
  if (stdfs::exists(dirname))
    for (const auto &direntry : stdfs::directory_iterator{dirname}) {
      if (stdfs::is_directory(direntry.path()))
        continue;
      std::string extension = toLower(direntry.path().extension().string());
      if (extension == ".so" or    //
          extension == ".dylib" or //
          extension == ".dll")
        filenames.push_back(direntry.path().string());
    }
  return filenames;
}

} // namespace d5
