#pragma once

#include <gtest/gtest.h>

#include "dirsig5/base/ScopeGuard.h"

#if USE_BOOST_FILESYSTEM
#include <boost/filesystem.hpp>
namespace stdfs = boost::filesystem;
using stdfs_error_code = boost::system::error_code;
#else
#include <filesystem>
namespace stdfs = std::filesystem;
using stdfs_error_code = std::error_code;
#endif

namespace d5 {

/// Test whether the given path exists
inline ::testing::AssertionResult fileExists(const char* path) {
  // check if file exists and if yes: Is it really a file and no directory?
  if (stdfs::exists(path)) {
    if (stdfs::is_regular_file(path)) {
      return ::testing::AssertionSuccess();
    } else {
      return ::testing::AssertionFailure()
             << "\"" << path << "\" exists but is not a file";
    }
  } else {
    return ::testing::AssertionFailure() << "\"" << path << "\" does not exist";
  }
}

/// Removes the given file
inline void RemoveFile(const char* file) {
  stdfs_error_code ec;
  stdfs::remove(file, ec);
}

/// \overload
inline void RemoveFile(const std::string& file) {
  stdfs_error_code ec;
  stdfs::remove(file, ec);
}

/// Makes a scope guard that removes the given file
inline auto make_rm_file_guard(const std::string& file) {
  return make_guard([=]() { RemoveFile(file); });
}

}  // namespace d5
