/** Unit tests for functional.h
 *
 *  \file FunctionalTest.cpp
 */

#include "dirsig5/base/Functional.h"

#include <memory>

#include <gtest/gtest.h>

using namespace d5;

class A {
public:
  A(int a) : a_(a) {}

  int foo1(const int &a, std::unique_ptr<int> b, int &c) {
    c = a_;
    return a_ + a + *b;
  }

  void foo2(int &a) const { a = a_; }

private:
  int a_;
};

TEST(FunctionalTest, BindMemFnTest) {
  A a(5);
  const A &cref = a;

  auto f1 = BindMemFn(&A::foo1, &a);
  auto f2 = BindMemFn(&A::foo2, &cref);

  int b;
  EXPECT_EQ(f1(4, std::make_unique<int>(6), b), 15);
  EXPECT_EQ(b, 5);

  int c;
  f2(c);
  EXPECT_EQ(c, 5);
}

int freeFn(int a) { return a; }

struct FuncObj {
  int operator()(int a) const { return a; };
};

constexpr auto globalLambda = [](int a) { return a; };

TEST(FunctionalTest, BindFnTest) {
  auto localLambda = [](int a) { return a; };

  EXPECT_EQ(BindFn(freeFn)(2), 2);
  EXPECT_EQ(BindFn(FuncObj{})(2), 2);
  EXPECT_EQ(BindFn(globalLambda)(2), 2);
  EXPECT_EQ(BindFn(localLambda)(2), 2);
}
