/** Unit test for NamedParameterStore.h
 *
 *  \file NamedParameterStoreTest.cpp
 */

#include "dirsig5/base/NamedParameterStore.h"

#include <string>
#include <vector>

#include <gtest/gtest.h>

using namespace d5;

struct Foo {
  int i;
};

#define TEST_NAMES(X)                                                          \
  X(a, int)                                                                    \
  X(b, std::string)                                                            \
  X(c, std::vector<double>)                                                    \
  X(d, Foo)

#define X_ENUM_DECL(n, t) n,
#define X_TYPE_TRAIT(n, t)                                                     \
  template <> struct d5::NamedParameterStoreTraits<Name, Name::n> {            \
    using type = t;                                                            \
  };

enum class Name { TEST_NAMES(X_ENUM_DECL) };

TEST_NAMES(X_TYPE_TRAIT)

TEST(NamedParameterStoreTest, TestCase1) {
  NamedParameterStore<Name> s;
  ASSERT_TRUE(s.empty());

  s.set<Name::a>(5);
  s.set<Name::b>("hello there");
  s.set<Name::c>({0, 1.5, 3});
  s.set<Name::d>({6});

  EXPECT_EQ(s.at<Name::a>(), 5);
  EXPECT_EQ(s.at<Name::b>(), "hello there");
  std::vector<double> cTruth{0, 1.5, 3};
  EXPECT_EQ(s.at<Name::c>(), cTruth);
  EXPECT_EQ(s.at<Name::d>().i, 6);

  s.at<Name::a>() = 7;
}
