#pragma once

#include <random>

#include <gtest/gtest.h>

#include "dirsig5/base/Constants.h"
#include "dirsig5/base/Pcg.h"

// Class derived from testing::Test to create a fixture
class SpectralFixture : public testing::Test {
protected:
  SpectralFixture(void)
      : seed(testing::UnitTest::GetInstance()->random_seed()), engine(seed) {}

  template <class VectorType>
  VectorType genWavelengthVector(double min, double max, size_t length) {
    VectorType v(length);

    if (length == 1) {
      v[0] = 0.5 * (max + min);
      return v;
    }

    double step = (max - min) / (length - 1);
    for (size_t ii = 0; ii < length; ii++) {
      v[ii] = min + step * ii;
    }
    return v;
  }

  template <class VectorType>
  VectorType genRandomVector(size_t length) {
    VectorType v(length);
    for (size_t ii = 0; ii < length; ii++) {
      v[ii] = randf(engine);
    }
    return v;
  }

  size_t randomLength(void) { return (randi(engine) % 1000) + 1; }

  void randomLimits(double& min, double& max) {
    double a = (100 - 0.2) * randf(engine) + 0.2;
    double b = a;
    while (b == a) {
      b = (100 - 0.2) * randf(engine) + 0.2;
    }
    if (a < b) {
      min = a;
      max = b;
      return;
    }
    min = b;
    max = a;
    return;
  }

  size_t randomIndex(size_t length) { return randi(engine) % length; }

private:
  int seed;
  d5::RandEngine engine;
  std::uniform_real_distribution<float> randf;
  std::uniform_int_distribution<int> randi;
};
