#include <gtest/gtest.h>

#include "CLStdComparisons.h"
#include "dirsig5/base/State.h"

using namespace d5;

template <typename T> constexpr T kEps = 1e-6;

TEST(SpectralRangesTest, TestEmpty) {
  UniformlySampledSpectralRanges ranges;
  EXPECT_TRUE(ranges.wavelengths().empty());
}

TEST(SpectralRangesTest, TestTrivial) {
  UniformlySampledSpectralRanges ranges;

  ranges.emplace(0.4, 0.5, 0.05);
  EXPECT_TRUE(IsNear(ranges.wavelengths<float>(),
                     std::vector<float>{0.4, 0.45, 0.5}, kEps<float>));

  ranges.emplace(0.1, 0.3, 0.05);
  EXPECT_TRUE(
      IsNear(ranges.wavelengths<float>(),
             std::vector<float>{0.1, 0.15, 0.2, 0.25, 0.3, 0.4, 0.45, 0.5},
             kEps<float>));
}

TEST(SpectralRangesTest, TestEqualBounds) {
  UniformlySampledSpectralRanges ranges;

  ranges.emplace(0.1, 0.2, 0.05);
  ranges.emplace(0.2, 0.3, 0.02);
  EXPECT_TRUE(
      IsNear(ranges.wavelengths(),
             std::vector<double>{0.1, 0.15, 0.2, 0.22, 0.24, 0.26, 0.28, 0.3},
             kEps<double>));
}

TEST(SpectralRangesTest, TestOverlappingSameRes) {
  UniformlySampledSpectralRanges ranges;
  ranges.emplace(0.2, 0.4, 0.1);
  ranges.emplace(0.1, 0.5, 0.1);
  ranges.emplace(0.2, 0.4, 0.1);
  ranges.emplace(0.2, 0.4, 0.1);
  ranges.emplace(0.4, 0.7, 0.1);
  EXPECT_TRUE(IsNear(ranges.wavelengths(),
                     std::vector<double>{0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7},
                     kEps<double>));
}

TEST(SpectralRangesTest, TestOverlappingDiffernetRes) {
  UniformlySampledSpectralRanges ranges;

  ranges.emplace(0.1, 0.2, 0.05);
  ranges.emplace(0.1, 0.2, 0.02);
  EXPECT_TRUE(IsNear(ranges.wavelengths(),
                     std::vector<double>{0.1, 0.12, 0.14, 0.16, 0.18, 0.2},
                     kEps<double>));

  ranges.emplace(0.15, 0.2, 0.01);
  EXPECT_TRUE(IsNear(
      ranges.wavelengths(),
      std::vector<double>{0.1, 0.12, 0.14, 0.15, 0.16, 0.17, 0.18, 0.19, 0.2},
      kEps<double>));

  ranges.emplace(0.05, 0.25, 0.05);
  EXPECT_TRUE(IsNear(ranges.wavelengths(),
                     std::vector<double>{0.05, 0.1, 0.12, 0.14, 0.15, 0.16,
                                         0.17, 0.18, 0.19, 0.2, 0.25},
                     kEps<double>));

  ranges.emplace(0.06, 0.36, 0.1);
  EXPECT_TRUE(IsNear(ranges.wavelengths(),
                     std::vector<double>{0.05, 0.1, 0.12, 0.14, 0.15, 0.16,
                                         0.17, 0.18, 0.19, 0.2, 0.25, 0.35},
                     kEps<double>));
}
