#include <thread>

#include <gtest/gtest.h>

#include "dirsig5/base/Timer.h"

using d5::Timer;

TEST(TimerTest, TimerTest) {
  Timer timer;

  EXPECT_FALSE(timer.isRunning());

  EXPECT_TRUE(timer.start());
  EXPECT_TRUE(timer.isRunning());
  std::this_thread::sleep_for(std::chrono::milliseconds(2));
  EXPECT_TRUE(timer.stop());

  EXPECT_FALSE(timer.isRunning());

  EXPECT_LT(timer.duration(Timer::kHours), 1);
  EXPECT_LT(timer.duration(Timer::kMinutes), 1);
  EXPECT_LT(timer.duration(Timer::kSeconds), 1);
  EXPECT_GT(timer.duration(Timer::kMilliseconds), 2);

  EXPECT_EQ(timer.format("%h hours"), "0 hours");
  EXPECT_EQ(timer.format("%m minutes"), "0 minutes");
  EXPECT_EQ(timer.format("seconds: %s"), "seconds: 0");
  EXPECT_EQ(timer.format("%hh %mm %ss"), "0h 0m 0s");

  auto result = timer.format("%v");
  EXPECT_GT(result.size(), 0u);

  result = timer.format("%.3s");
  EXPECT_EQ(result.size(), 5u);

  double parsed = std::stod(result);
  EXPECT_NEAR(parsed, timer.duration(Timer::kSeconds), 5e-4);

  timer.start();
  EXPECT_TRUE(timer.isRunning());
  EXPECT_FALSE(timer.start());
  EXPECT_DOUBLE_EQ(timer.duration(Timer::kSeconds), 0);
  EXPECT_EQ(timer.format("%.3s"), "");
  EXPECT_TRUE(timer.stop());
  EXPECT_FALSE(timer.stop());
  EXPECT_DOUBLE_EQ(timer.duration(static_cast<Timer::TimeUnit>(10000)), 0);
}
