#include <array>
#include <vector>

#include <gtest/gtest.h>

#include "dirsig5/base/Weather.h"

using namespace d5;

TEST(CLWeatherTest, UsageTest) {
  // Test the with*() setters
  Weather wth;
  wth.airTemperature = Temperature::fromCelsius(0);
  wth.airPressureMbar = 12;
  wth.relativeHumidity = 0.6;
  wth.windSpeedMS = 4;
  wth.directInsolationWM2 = 10;
  wth.diffuseInsolationWM2 = 5;

  // Test approximated/real dew point
  EXPECT_NEAR(wth.dewPoint().get<Temperature::Celsius>(), -8, 1e-10);
  auto dp = Temperature::fromCelsius(-6.8);
  wth.withDewPoint(dp);
  EXPECT_EQ(wth.dewPoint(), dp);
  wth.clearDewPoint();
  EXPECT_NEAR(wth.dewPoint().get<Temperature::Celsius>(), -8, 1e-10);
  wth.setDewPoint(dp);
  EXPECT_EQ(wth.dewPoint(), dp);
}
