#include "dirsig5/brdf/Lambertian.h"

#include "Unzip.h"
#include "dirsig5/base/SpectralVector.h"

namespace d5::brdf {

std::vector<double> Lambertian::sample(
    double illumZenithRad, double, double viewZenithRad, double) const {
  double cosThetaO = std::cos(viewZenithRad);
  double cosThetaI = std::cos(illumZenithRad);
  if (cosThetaO < 0) {
    cosThetaO *= -1;
    cosThetaI *= -1;
  }
  return cosThetaI >= 0 ? reflectanceOverPi_ : transmittanceOverPi_;
}

void Lambertian::init() {
  using Params = Parameters::SpectralParameters;
  std::vector<double> paramsWave;
  std::vector<double> paramsReflectance;
  std::vector<double> paramsTransmittance;
  Unzip(
      params_.spectral, {{&Params::wavelengthMicrons, &paramsWave},
                         {&Params::reflectance, &paramsReflectance},
                         {&Params::transmittance, &paramsTransmittance}});
  reflectanceOverPi_.clear();
  transmittanceOverPi_.clear();
  interpolate(
      paramsWave, paramsReflectance, wavelengthMicrons(), reflectanceOverPi_);
  interpolate(
      paramsWave, paramsTransmittance, wavelengthMicrons(),
      transmittanceOverPi_);
  for (double& each : reflectanceOverPi_) each *= M_1_PI;
  for (double& each : transmittanceOverPi_) each *= M_1_PI;
}

std::vector<double> Lambertian::doSimplifySpectralBrdf(
    std::vector<double>& nadirDHR) {
  nadirDHR = reflectanceOverPi_;
  for (double& each : nadirDHR) each *= M_PI;  // Right?
  return {params_.spectral.front().wavelengthMicrons};
}

void Lambertian::doGetHashInputs(const HashCallback& callback) const {
  // Version number (used as hash salt, update on any implementation changes)
  static constexpr int kVersion = 1;
  AddHashData(kVersion, callback);
  AddHashData(params_.spectral, callback);
}

}  // namespace d5::brdf
