#include "dirsig5/brdf/Ward.h"

#include "Unzip.h"
#include "dirsig5/base/Geometry.h"
#include "dirsig5/base/SpectralVector.h"

namespace d5::brdf {

std::vector<double> Ward::sample(
    double thetaI, double phiI, double thetaO, double phiO) const {
  std::vector<double> result(wavelengthMicrons().size());

  // Compute direction vectors.
  Vector omegaI = {
      std::sin(thetaI) * std::cos(phiI),  //
      std::sin(thetaI) * std::sin(phiI),  //
      std::cos(thetaI)};
  Vector omegaO = {
      std::sin(thetaO) * std::cos(phiO),  //
      std::sin(thetaO) * std::sin(phiO),  //
      std::cos(thetaO)};

  // View direction always in upper hemisphere.
  if (omegaO[2] < 0) {
    omegaO[2] *= -1;
    omegaI[2] *= -1;
  }

  Vector omegaH = (omegaO + omegaI).normalized();
  if (omegaO[2] > 0 and omegaI[2] > 0 and omegaH[2] > 0) {
    double tanThetaH = std::tan(std::acos(omegaH[2]));
    double tan2ThetaH = tanThetaH * tanThetaH;
    for (size_t index = 0; index < result.size(); index++) {
      double sigma2 = sigma_[index] * sigma_[index];
      result[index] = 0.25 * M_1_PI * specular_[index] /
                      (std::sqrt(omegaI[2] * omegaO[2]) * sigma2);
      result[index] *= std::exp(-tan2ThetaH / sigma2);
      result[index] += diffuse_[index] * M_1_PI;
    }
  }
  return result;
}

void Ward::init() {
  using Params = Parameters::SpectralParameters;
  std::vector<double> paramsWave;
  std::vector<double> paramsSigma;
  std::vector<double> paramsDiffuse;
  std::vector<double> paramsSpecular;
  Unzip(
      params_.spectral, {{&Params::wavelengthMicrons, &paramsWave},
                         {&Params::sigma, &paramsSigma},
                         {&Params::diffuse, &paramsDiffuse},
                         {&Params::specular, &paramsSpecular}});
  sigma_.clear();
  diffuse_.clear();
  specular_.clear();
  interpolate(paramsWave, paramsSigma, wavelengthMicrons(), sigma_);
  interpolate(paramsWave, paramsDiffuse, wavelengthMicrons(), diffuse_);
  interpolate(
      paramsWave, paramsSpecular, wavelengthMicrons(), specular_);
}

std::vector<double> Ward::doSimplifySpectralBrdf(
    std::vector<double>& nadirDHR) {
  // TODO
  return {};
}

void Ward::doGetHashInputs(const HashCallback& callback) const {
  // Version number (used as hash salt, update on any implementation changes)
  static constexpr int kVersion = 1;
  AddHashData(kVersion, callback);
  AddHashData(params_.spectral, callback);
}

}  // namespace d5::brdf
