#include "dirsig5/d4legacy/geometry/ControlTable.h"

#include <fstream>

#include "dirsig5/base/Error.h"
#include "dirsig5/base/ErrorIO.h"
#include "dirsig5/base/StringUtils.h"
#include "spdlog/fmt/fmt.h"

namespace d5::d4legacy {

ControlTable ReadControlTableFromFile(const std::string& filename) {
  auto in = OpenIfstream(filename);

  ControlTable ct;
  std::string line;
  size_t index = 0, lineNum = 0;
  while (std::getline(in, line)) {
    lineNum++;
    auto sl = TokenList(line, ",");
    if (sl.empty()) continue;

    if (index++ == 0) {
      for (auto s : sl)
        ct.tagIndex.emplace_back(std::string(s.substr(1, s.size() - 2)));
    } else {
      if (static_cast<size_t>(sl.size()) != (ct.tagIndex.size() + 1))
        DIRSIG_THROW_FMT(
          "line {}: invalid control table, expected {} columns, "
          "found {}",
          lineNum, ct.tagIndex.size() + 1, sl.size());

      bool first = true;
      for (auto s : sl) {
        if (first) {
          first = false;
          ct.times.push_back(ParseNumber<float>(s.substr(1, s.size() - 2)));
        } else
          ct.states.push_back(ParseNumber<float>(s.substr(1, s.size() - 2)));
      }
    }
  }

  return ct;
}

}  // namespace d5::d4legacy
