#include "dirsig5/d4legacy/image_io/TemporalImages.h"

#include <fstream>
#include <sstream>

#include "dirsig5/base/Error.h"
#include "dirsig5/base/ErrorIO.h"
#include "dirsig5/base/Filesystem.h"
#include "dirsig5/base/StringUtils.h"
#include "dirsig5/base/Logger.h"

namespace d5::d4legacy {

std::vector<TimeVaryingImageEntry> ReadTilFile(const std::string& filename) {
  DIRSIG_ERROR_TRACE("reading TIL file \"" + filename + "\"", {
    auto in = OpenIfstream(filename);
    std::vector<TimeVaryingImageEntry> res;

    bool usedRelative = false;
    bool usedAbsolute = false;
    std::string line;
    while (std::getline(in, line)) {
      line = trimmed(line);
      size_t sep = line.find_first_of(' ');
      DIRSIG_THROW_IF(
        sep == std::string::npos,
        "TIL file lines should be of the form:\ntime filename");
      DateTime time;
      double timeSec = 0;
      auto timeStr = line.data();
      auto timeStrEnd = line.data() + sep;
      if (auto status = ParseNumber(timeStr, &timeStrEnd, timeSec);
          status == NumericParseStatus::Success &&
          timeStrEnd == line.data() + sep) {
        usedRelative = true;
        time.setRelativeSeconds(timeSec);
      } else {
        usedAbsolute = true;
        time.setIso8601(line.substr(0, sep));
      }
      std::string imgFilename = trimmed(line.substr(sep + 1));
      auto& r = res.emplace_back();
      r.time = time;
      r.filename =
        (stdfs::absolute(filename).parent_path() / imgFilename).string();
      DIRSIG_THROW_IF_FMT(
        !stdfs::exists(r.filename),
        "referenced file \"{}\" does not exist relative "
        "to the TIL file",
        imgFilename);
    }
    DIRSIG_THROW_IF(
      usedRelative && usedAbsolute,
      "Expected TIL file to use either all relative times or all absolute (ISO "
      "8601) times.");

    return res;
  });
}

}  // namespace d5::d4legacy
