/** Unit test for SimpleSpectralData.h
 *
 *  \file SimpleSpectralDataTest.cpp
 */

#include "dirsig5/d4legacy/mat/SimpleSpectralData.h"

#include <gtest/gtest.h>

#include "CLTemporaryFile.h"

TEST(SimpleSpectralDataTest, TxtTest) {
  auto filename = TempFilename(".txt");
  std::ofstream ofs(filename);
  ASSERT_TRUE(ofs.is_open());
  ofs << R"(
1.0 0.125
1.5 0.5
# A comment
2.125 0.75
# Another comment
4.0 1.0
)";
  ofs.close();

  d5::d4legacy::SimpleSpectralData ssd;
  ASSERT_NO_THROW(
      ssd = d5::d4legacy::ReadSimpleSpectralDataFromFile(
          filename.c_str(), d5::d4legacy::SimpleSpectralData::Format::Text));
  EXPECT_EQ(ssd.wavelengthsMicron[0], 1.0f);
  EXPECT_EQ(ssd.wavelengthsMicron[1], 1.5f);
  EXPECT_EQ(ssd.wavelengthsMicron[2], 2.125f);
  EXPECT_EQ(ssd.wavelengthsMicron[3], 4.0f);
  EXPECT_EQ(ssd.spectrum[0], 0.125f);
  EXPECT_EQ(ssd.spectrum[1], 0.5f);
  EXPECT_EQ(ssd.spectrum[2], 0.75f);
  EXPECT_EQ(ssd.spectrum[3], 1.0f);
}

TEST(SimpleSpectralDataTest, XmlTest) {
  auto filename = TempFilename(".xml");
  std::ofstream ofs(filename);
  ASSERT_TRUE(ofs.is_open());
  ofs << R"(
<spectraldata spectralunits="microns">
<point><spectralpoint>1.0</spectralpoint><value>0.125</value></point>
<point><spectralpoint>1.5</spectralpoint><value>0.5</value></point>
<point><spectralpoint>2.125</spectralpoint><value>0.75</value></point>
<point><spectralpoint>4.0</spectralpoint><value>1.0</value></point>
</spectraldata>
)";
  ofs.close();

  d5::d4legacy::SimpleSpectralData ssd;
  ASSERT_NO_THROW(
      ssd = d5::d4legacy::ReadSimpleSpectralDataFromFile(
          filename.c_str(), d5::d4legacy::SimpleSpectralData::Format::XML));
  EXPECT_EQ(ssd.wavelengthsMicron[0], 1.0f);
  EXPECT_EQ(ssd.wavelengthsMicron[1], 1.5f);
  EXPECT_EQ(ssd.wavelengthsMicron[2], 2.125f);
  EXPECT_EQ(ssd.wavelengthsMicron[3], 4.0f);
  EXPECT_EQ(ssd.spectrum[0], 0.125f);
  EXPECT_EQ(ssd.spectrum[1], 0.5f);
  EXPECT_EQ(ssd.spectrum[2], 0.75f);
  EXPECT_EQ(ssd.spectrum[3], 1.0f);
}
