/** Implementation file for EndianSupport.h
 *
 *  \file EndianSupport.cpp
 */

#include "dirsig5/d4legacy/misc/EndianSupport.h"

namespace d5::d4legacy {

EndianSupport::ByteOrder EndianSupport::nativeByteOrder() {
  union probe {
    unsigned int num;
    unsigned char bytes[sizeof(unsigned int)];
  };

  // initialze the first member of p with an unsigned 1
  probe p = {1U};

  ByteOrder foundByteOrder;

  // on a big endian architecture, p.bytes[0] equals 0,
  // on a little endian architecture, p.bytes[0] equals 1
  if (p.bytes[0] == 0U) {
    foundByteOrder = BigEndian;
  } else if (p.bytes[0] == 1U) {
    foundByteOrder = LittleEndian;
  } else {
    foundByteOrder = UnknownEndian;
  }

  return foundByteOrder;
}

void EndianSupport::swapTwoByteType(char* data) {
  char temp = *(data + 1);
  *(data + 1) = *data;
  *data = temp;
}

void EndianSupport::swapFourByteType(char* data) {
  char temp = *(data + 3);
  *(data + 3) = *data;
  *data = temp;
  temp = *(data + 2);
  *(data + 2) = *(data + 1);
  *(data + 1) = temp;
}

void EndianSupport::swapEightByteType(char* data) {
  char temp = *(data + 7);
  *(data + 7) = *data;
  *data = temp;

  temp = *(data + 6);
  *(data + 6) = *(data + 1);
  *(data + 1) = temp;

  temp = *(data + 5);
  *(data + 5) = *(data + 2);
  *(data + 2) = temp;

  temp = *(data + 4);
  *(data + 4) = *(data + 3);
  *(data + 3) = temp;
}
}  // namespace d5::d4legacy
