/** Interface file for LineReader
 *
 *  \file LineReader.h
 */

#include <istream>
#include <string>

#include "spdlog/fmt/fmt.h"
#include "dirsig5/base/Error.h"

namespace d5::d4legacy {

/// A simple helper struct for reading files line-by-line
struct LineReader {
  LineReader(std::istream& is) : stream(is) {}

  std::istream& stream;
  std::string line;
  std::size_t lineno = 0;

  bool nextLineOptional() {
    if (std::getline(stream, line)) {
      lineno++;
      return true;
    } else {
      return false;
    }
  }

  void nextLineRequired() {
    if (!nextLineOptional()) fail("unexpected EOF");
  }

  void fail(const char* what) { DIRSIG_THROW_FMT("line {}: {}", lineno, what); }
};
}
