#include "dirsig5/d4legacy/platform/FocalPlane.h"

using namespace std::string_view_literals;

namespace d5::d4legacy::platform {

void FocalPlane::ReadXML(const xml::Node& node) {
  auto capture{xml::requiredChild(node, "capturemethod")};
  *this = {};
  if (node.attribute("enabled")) enabled = node.attribute("enabled").as_bool();
  name = node.attribute("name").as_string();
  captureName = capture.attribute("name").as_string();
  captureType = capture.attribute("type").as_string();
  imageFile.ReadXML(xml::requiredChild(capture, "imagefile"));
  if (auto subnode = node.child("detectorarray")) {
    detectorArray = DetectorArray();
    detectorArray->ReadXML(subnode);
  }
  if (auto subnode = capture.child("detectormodel")) {
    detectorModel = DetectorModel();
    detectorModel->ReadXML(subnode);
  }
  if (auto subnode = capture.child("spectralresponse")) {
    spectralResponse = SpectralResponse();
    spectralResponse->ReadXML(subnode);
  }
  if (auto subnode = capture.child("gainresponse")) {
    gainResponse = GainResponse();
    gainResponse->ReadXML(subnode);
  }
  if (auto subnode = capture.child("spatialresponse")) {
    spatialSubsamplesX = subnode.child("xsamples").text().as_int();
    spatialSubsamplesY = subnode.child("ysamples").text().as_int();
  }
  if (auto subnode = capture.child("samplestrategy")) {
    hypersamplingMultiplier =
      xml::readNumber(subnode, "hypersamplingmultiplier", 1.0);
  }
  if (auto subnode = capture.child("temporalintegration")) {
    integrationTime = subnode.child("time").text().as_double();
    useTdi = subnode.attribute("tdi").as_bool();
  }
  if (auto subnode = node.child("truthcollection")) {
    truthCollection = TruthCollection();
    truthCollection->ReadXML(subnode);
  }
  if (capture.child("pulsecompression").text().as_string() == "enabled"sv) {
    compressionLevel = capture.child("compressionlevel").text().as_string();
  }
  if (auto subnode = capture.child("activearea")) {
    extras.activeArea =
      Extras::ActiveArea{subnode.child("image").text().as_string()};
  }
  if (auto subnode = capture.child("straylight")) {
    extras.strayLight = Extras::StrayLight{
      subnode.child("grid").text().as_string(),
      subnode.child("fraction").text().as_double()};
  }
  if (auto subnode = capture.child("psf")) {
    extras.psf = Extras::Psf{
      subnode.child("image").text().as_string(),
      subnode.child("grid").text().as_string(),
      subnode.child("width").text().as_double(),
      subnode.child("scale").text().as_double()};
  }
  // TODO processing
  if (auto subnode = capture.child("convergence")) {
    extras.convergence = Extras::Convergence{
      subnode.child("threshold").text().as_double(0),
      subnode.child("minimumsamples").text().as_int(1),
      subnode.child("maximumsamples").text().as_int(1),
      subnode.child("maxnodes").text().as_int(5)};
  }
}

}  // namespace d5::d4legacy::platform
