#include "dirsig5/d4legacy/platform/Mount.h"
#include "dirsig5/base/Logger.h"

namespace d5::d4legacy::platform {

void Mount::ReadXML(const xml::Node& node) {
  auto data = xml::requiredChild(node, "data");
  name = xml::requiredAttribute(node, "name").as_string();
  type = xml::requiredAttribute(node, "type").as_string();
  if (type == "commanded" || type == "tabulated") {
    typeVariant.emplace<CommandedOrTabulated>().ReadXML(data);
  } else if (type == "line" || type == "whisk") {
    typeVariant.emplace<LineOrWhisk>().ReadXML(data);
  } else if (type == "lemniscate") {
    typeVariant.emplace<Lemniscate>().ReadXML(data);
  } else if (type == "python") {
    typeVariant.emplace<Python>().ReadXML(data);
  } else if (type == "static") {
    typeVariant.emplace<Static>().ReadXML(data);
  } else if (type == "tracking") {
    typeVariant.emplace<Tracking>().ReadXML(data);
  } else {
    DIRSIG_THROW_FMT(
      "Unknown mount type '{}', expected 'commanded', 'lemniscate', 'line', "
      "'python', 'static', 'tabulated', 'tracking', or 'whisk'",
      type);
  }
}

}  // namespace d5::d4legacy::platform
