#include "dirsig5/d4legacy/sim/SimFile.h"

#include "dirsig5/base/Filesystem.h"
#include "dirsig5/d4legacy/xml/xml.h"
#include "dirsig5/base/StandardPaths.h"

namespace d5::d4legacy {

// Helper function for computing an absolute path
static std::string absFilename(
  const std::string& filename, const std::string& base) {
  if (filename.empty() or stdfs::path(filename).is_absolute())
    return filename;
  else
    return (stdfs::path(base) /  //
            stdfs::path(filename))
      .string();
}

std::unique_ptr<SimFile> SimFile::LoadFromXmlFile(const std::string& filename) {
  auto doc = xml::loadFile(filename);
  auto root = doc.document_element();

  // extract the path to the simulation file to resolve relative paths
  auto absPath = stdfs::absolute(filename).parent_path().string();
  auto getPath = [&](const char* tag) {
    return absFilename(
      StandardPaths::withReplacements(
        xml::requiredAttribute(xml::requiredChild(root, tag), "externalfile")
          .as_string()),
      absPath);
  };

  auto sim = std::make_unique<SimFile>();
  sim->filename = filename;
  sim->sceneFilename = getPath("scene");
  sim->atmosphereFilename = getPath("atmosphericconditions");
  sim->platformFilename = getPath("platform");
  sim->platformMotionFilename = getPath("platformmotion");
  sim->tasksFilename = getPath("tasklist");
  if (root.child("options")) sim->optionsFilename = getPath("options");
  return sim;
}

}  // namespace d5::d4legacy
