#include "dirsig5/d4legacy/sources/SourceInfo.h"

#include "dirsig5/base/Error.h"
#include "dirsig5/d4legacy/xml/xml.h"

using namespace std::string_view_literals;

namespace d5::d4legacy {

SimpleSpectralData ReadSourceIntensityFile(const std::string& filename) {
  auto ssd = ReadSimpleSpectralDataFromFile(
    filename.c_str(), SimpleSpectralData::Format::Text);
  // This is taken from FDLegacyPointSource::load(). Not sure why this is
  // needed because the units are W/sr/micron, but it is needed to match the
  // behavior of D4
  for (auto& i : ssd.spectrum) i /= 10000.0;
  return ssd;
}

std::vector<std::array<double, 3>> ReadPsdFile(const std::string& filename) {
  std::vector<std::array<double, 3>> components;
  auto doc = xml::loadFile(filename);
  auto root = doc.document_element();
  if (root.name() != "psd"sv)
    DIRSIG_THROW("not a PSD file (root element must be <psd>)");
  for (auto entry : root.children("entry")) {
    double phase = entry.child("phase").text().as_double();
    components.push_back(
      {xml::requiredChild(entry, "frequency").text().as_double(),
       xml::requiredChild(entry, "magnitude").text().as_double(), phase});
  }
  return components;
}

}  // namespace d5::d4legacy
