/** ENVI Header I/O for Endian.h
 *
 *  \file EndianIO.h
 */

#pragma once

#include "HeaderFieldIO.h"

namespace d5::envi {

template <>
struct header_field_io<Endian, void> {
  static void serialize(std::string& str, Endian val) {
    switch (val) {
      case Endian::Little:
        serialize_header_field(str, 0);
        break;
      case Endian::Big:
        serialize_header_field(str, 1);
        break;
      case Endian::Unknown:
        throw std::runtime_error("unknown endian value");
    }
    throw std::runtime_error("invalid endian value");
  }

  static void deserialize(const std::string& str, Endian& val) {
    uint8_t v;
    deserialize_header_field(str, v);
    if (v == 0) {
      val = Endian::Little;
    } else if (v == 1) {
      val = Endian::Big;
    } else {
      throw std::runtime_error("\"byte order\" must be either 0 or 1");
    }
  }
};
}
