/** Implementation file for ENVI image I/O
 *
 *  \file ImageIO.cpp
 */

#include "dirsig5/envi/ImageIO.h"

#include <tbb/parallel_for.h>

#include "dirsig5/envi/Endian.h"

void d5::envi::SwapEndian(ImageDestination &dst, const ReadParameters &params,
                          DType dtype) {
  auto byteSize = ByteSize(dtype);
  auto isComplex = IsComplex(dtype);
  if (isComplex)
    byteSize /= 2;
  if (byteSize == 1)
    return;
  if (byteSize != 2 && byteSize != 4 && byteSize != 8) {
    throw std::runtime_error("unsupported byte size to endian-swap");
  }

  auto swap = [&](char *ptr) {
    switch (byteSize) {
    case 2:
      swapEndian(*reinterpret_cast<uint16_t *>(ptr));
      break;
    case 4:
      swapEndian(*reinterpret_cast<uint32_t *>(ptr));
      break;
    case 8:
      swapEndian(*reinterpret_cast<uint64_t *>(ptr));
      break;
    }
  };

  if (dst.interleave() == Interleave::BIP) {
    std::size_t numValsInRow = params.stridedNumCols * params.numBands;
    if (isComplex)
      numValsInRow *= 2;

    tbb::parallel_for(std::size_t{0}, params.stridedNumRows,
                      [&](std::size_t r) {
                        auto *row = dst.row(0, r);
                        if (row)
                          for (size_t i = 0; i < numValsInRow; i++)
                            swap(row + i * byteSize);
                      });
  } else {
    std::size_t numValsInRow = params.stridedNumCols;
    if (isComplex)
      numValsInRow *= 2;

    tbb::parallel_for(std::size_t{0}, params.stridedNumRows,
                      [&](std::size_t r) {
                        for (size_t b = 0; b < params.numBands; b++) {
                          auto *row = dst.row(b, r);
                          if (row)
                            for (size_t i = 0; i < numValsInRow; i++)
                              swap(row + i * byteSize);
                        }
                      });
  }
}
