#include "dirsig5/envi/SpectralLibrary.h"

#include <gtest/gtest.h>

#include "CLFilesystemTest.h"
#include "CLTemporaryFile.h"
#include "dirsig5/base/DType.h"

template <typename T>
static std::vector<T> WriteTempFile(const std::string &filename) {
  std::vector<T> curve(8);
  for (int i = 0; i < 8; i++)
    curve[i] = T(i);
  {
    std::ofstream ofs(filename + ".hdr");
    ofs << "ENVI\n"
           "description = {None}\n"
           "samples = 8\n"
           "lines = 1\n"
           "bands = 1\n"
           "header offset = 0\n"
           "file type = ENVI Spectral Library\n"
           "data type = ";
    ofs << int(d5::dtype_v<T>) << '\n';
    ofs << "interleave = bsq\n"
           "byte order = 0\n"
           "wavelength units = Micrometers\n"
           "wavelength = {0.40, 0.41, 0.42, 0.43, 0.44, 0.45, 0.46, 0.47, "
           "0.48}\n"
           "reflectance scale factor = 1\n";
  }
  {
    std::ofstream ofs(filename, std::ios::binary | std::ios::out);
    ofs.write(reinterpret_cast<char *>(curve.data()), sizeof(T) * curve.size());
  }
  return curve;
}

TEST(SpectralLibraryTests, ReadInt16Test) {
  auto filename = d5::TempFilename(".img");
  auto curve = WriteTempFile<int16_t>(filename);
  d5::envi::SpectralLibrary spectralLibrary;
  EXPECT_NO_THROW(spectralLibrary.read(filename));
  EXPECT_EQ(spectralLibrary.spectra.size(), 1);
  EXPECT_EQ(spectralLibrary.spectra[0].size(), curve.size());
  for (size_t i = 0; i < curve.size(); i++)
    EXPECT_EQ(float(spectralLibrary[0][i]), curve[i]);
}

TEST(SpectralLibraryTests, ReadInt32Test) {
  auto filename = d5::TempFilename(".img");
  auto curve = WriteTempFile<int32_t>(filename);
  d5::envi::SpectralLibrary spectralLibrary;
  EXPECT_NO_THROW(spectralLibrary.read(filename));
  EXPECT_EQ(spectralLibrary.spectra.size(), 1);
  EXPECT_EQ(spectralLibrary.spectra[0].size(), curve.size());
  for (size_t i = 0; i < curve.size(); i++)
    EXPECT_EQ(float(spectralLibrary[0][i]), curve[i]);
}

TEST(SpectralLibraryTests, ReadFloat32Test) {
  auto filename = d5::TempFilename(".img");
  auto curve = WriteTempFile<float>(filename);
  d5::envi::SpectralLibrary spectralLibrary;
  EXPECT_NO_THROW(spectralLibrary.read(filename));
  EXPECT_EQ(spectralLibrary.spectra.size(), 1);
  EXPECT_EQ(spectralLibrary.spectra[0].size(), curve.size());
  for (size_t i = 0; i < curve.size(); i++)
    EXPECT_EQ(float(spectralLibrary[0][i]), curve[i]);
}
