#include "dirsig5/flex_motion/LocationEngines/Sgp4.h"

#include "Sgp4Facade.h"
#include "sgp4ext.h"
#include "sgp4io.h"
#include "sgp4unit.h"

namespace d5::flex_motion::LocationEngines {

struct Sgp4::Impl {
  elsetrec satRecord;
};

Sgp4::Sgp4() : pimpl_(new Impl()) {}

Sgp4::~Sgp4() { delete pimpl_; }

void Sgp4::doUpdateState(const StateDateTime &, const geolib::Facade *facade) {
  if (!geoLibFacade) { // TODO Calling twoline2rv more than once is a problem?
    geoLibFacade = facade;
    geolib::Location sceneOrigin(geoLibFacade);
    ecefToEnuTransform = geolib::Location::computeEcefToEnuMatrix(sceneOrigin);

    // Run the standard TLE parser included in SGP4
    char runType = 'c';
    char inputType = 'm';
    char opsMode = 'a';
    auto grav = wgs72;
    double dummyTime1 = 0;
    double dummyTime2 = 0;
    double dummyTime3 = 0;
    twoline2rv(tleLine1, tleLine2, runType, inputType, opsMode, grav,
               dummyTime1, dummyTime2, dummyTime3, pimpl_->satRecord);
  }
}

LocationState Sgp4::doGet(const DateTime &absoluteTime, double) const {
  // Run the calculation (output in TEME)
  double startDfe = absoluteTime.toJulianDate() - pimpl_->satRecord.jdsatepoch;
  double startMfe = startDfe * (60 * 24);
  double temePosition[3] = {};
  double temeVelocity[3] = {};
  auto gravConst = wgs72;
  pimpl_->satRecord.t = 0; // TODO Thread problem?
  sgp4(gravConst, pimpl_->satRecord, startMfe, temePosition, temeVelocity);

  // Convert the position and velocity from TEME to ECEF
  EcefCoord ecefPosition, ecefVelocity;
  Sgp4Facade::temeToEcef(gstime(absoluteTime.toJulianDate()), temePosition,
                         temeVelocity, ecefPosition, ecefVelocity);

  // If we want ECEF, return it now
  if (frame == d5::geolib::Location::EcefType) {
    return {ecefPosition, ecefVelocity};
  }

  // Make sure we have a geolib facade so we can convert to ENU
  DIRSIG_THROW_IF(
      geoLibFacade == nullptr,
      "SGP4 attempted to convert to ENU without a valid geolib facade!");

  // Convert the ECEF location to Scene ENU
  Point scenePosition = geoLibFacade->from(ecefPosition);
  Vector sceneVelocity = ecefToEnuTransform * Point(ecefVelocity);
  return {scenePosition, sceneVelocity};
}

} // namespace d5::flex_motion::LocationEngines
