#pragma once

#include <string>

#include "dirsig5/base/DateTime.h"
#include "dirsig5/flex_motion/Export.h"
#include "dirsig5/geolib/Location.h"

namespace d5::flex_motion {

class D5_FM_EXPORT Sgp4Facade {
public:
  /**
   * Run the SGP4 orbit propagation model using the supplied TLE
   *
   * \param desiredTime The date/time for the satellite position and velocity
   * \param tleLine1    The first line of the two-line element.
   * \param tleLine2    The second line of the two-line element.
   * \param position    The computed position vector in ECEF coordinates.
   * \param velocity    The computed velocity vector in ECEF coordinates.
   */
  void run(
      const DateTime& desiredTime,
      char* tleLine1,
      char* tleLine2,
      EcefCoord& position,
      EcefCoord& velocity);

  /**
   * This function converts the supplied position and velocity from
   * true equator, mean equinox (TEME) coordinates to earth center,
   * earth fixed (ECEF) coordinates.  This involves handling both the
   * difference in earth centered, inertial (ECI) coordinates systems
   * (TEME vs. J2000) and the projection onto the rotating ECEF coordinate
   * system for the request time.
   */
  static void temeToEcef(
      double siderealTime,
      double temePosition[3],
      double temeVelocity[3],
      EcefCoord& ecefPosition,
      EcefCoord& ecefVelocity);
};

}  // namespace d5::flex_motion
