#include "dirsig5/flex_motion/LocationEngines/Waypoints.h"

#include "dirsig5/base/LerpUtils.h"

namespace d5::flex_motion::LocationEngines {

LocationState Waypoints::doGet(const DateTime &, double relativeTime) const {
  if (keyframes.size() == 0)
    return {{0, 0, 0}, {0, 0, 0}};
  if (keyframes.size() == 1)
    return {keyframes.front().location, {0, 0, 0}};
  const Keyframe &keyframe0 = keyframes.front();
  const Keyframe &keyframeN = keyframes.back();
  double timeSec0 = keyframe0.timeSec;
  double timeSecN = keyframeN.timeSec;
  if (loop)
    relativeTime = wrapFloat(relativeTime, timeSec0, timeSecN);
  auto [keyframeA, keyframeB] =
      findLerpIterators(keyframes.begin(), keyframes.end(), relativeTime,
                        [](const Keyframe &keyframe, double timeSec) {
                          return keyframe.timeSec < timeSec;
                        });
  if (relativeTime < timeSec0)
    return {keyframe0.location};
  if (relativeTime > timeSecN)
    return {keyframeN.location};
  double fac =
      findLerpFactor(relativeTime, keyframeA->timeSec, keyframeB->timeSec);
  Point location = (1 - fac) * keyframeA->location + fac * keyframeB->location;
  if (keyframeA == keyframeB) {
    if (keyframeA == keyframes.begin())
      ++keyframeB;
    else
      --keyframeA;
  }
  Vector velocity = (keyframeB->location - keyframeA->location) /
                    (keyframeB->timeSec - keyframeA->timeSec);
  return {location, velocity};
}

} // namespace d5::flex_motion::LocationEngines
