#include "dirsig5/flex_motion/Motion.h"

namespace d5::flex_motion {

DualQuaternion Motion::getRelative(double relativeTime) const {
  return get(cache.referenceDateTime + cache.elapsedTime + relativeTime);
}

DualQuaternion Motion::get(const DateTime &dateTime) const {
  DualQuaternion transform = doGet(dateTime);
  bool anyJitter = false;
  for (size_t index = 0; index < 3; index++)
    if (locationJitter[index] or rotationJitter[index]) {
      anyJitter = true;
      break;
    }
  if (anyJitter) {
    Point location = transform.pointForward({0, 0, 0});
    Quaternion rotation = transform.real;
    double relativeTime =
        (cache.referenceDateTime.makeAbsolute(dateTime) -
         cache.referenceDateTime.makeAbsolute(jitterStartTime))
            .getTotalSeconds();
    for (size_t index = 0; index < 3; index++)
      if (locationJitter[index])
        location[index] += locationJitter[index]->generate(relativeTime);
    for (size_t index = 0; index < 3; index++)
      if (rotationJitter[index])
        rotation = rotation * Eigen::AngleAxisd(
                                  rotationJitter[index]->generate(relativeTime),
                                  Vector::Unit(index));
    transform = DualQuaternion(rotation, location);
  }
  return transform;
}

} // namespace d5::flex_motion
