#include "dirsig5/flex_motion/Motions/DataDriven.h"

#include "dirsig5/base/LerpUtils.h"

namespace d5::flex_motion::Motions {

void DataDriven::doUpdateState(const StateDateTime &, const geolib::Facade *) {
  std::sort(keyframes.begin(), keyframes.end(),
            [](const Keyframe &lhs, const Keyframe &rhs) {
              return lhs.timeSec < rhs.timeSec;
            });
}

DualQuaternion DataDriven::doGet(const DateTime &dateTime) const {
  if (keyframes.empty())
    return {};
  if (keyframes.size() == 1)
    return keyframes[0].transform;
  double relativeTime = (cache.referenceDateTime.makeAbsolute(dateTime) -
                         cache.referenceDateTime.makeAbsolute(startDateTime))
                            .getTotalSeconds();
  const Keyframe &keyframe0 = keyframes.front();
  const Keyframe &keyframeN = keyframes.back();
  double timeSec0 = keyframe0.timeSec;
  double timeSecN = keyframeN.timeSec;
  if (loop)
    relativeTime = wrapFloat(relativeTime, timeSec0, timeSecN);
  auto [keyframeA, keyframeB] =
      findLerpIterators(keyframes.begin(), keyframes.end(), relativeTime,
                        [](const Keyframe &keyframe, double timeSec) {
                          return keyframe.timeSec < timeSec;
                        });
  if (keyframeA == keyframeB or //
      keyframeA->timeSec == keyframeB->timeSec)
    return keyframeA->transform;
  return ScLERP(
      keyframeA->transform, //
      keyframeB->transform, //
      findLerpFactor(relativeTime, keyframeA->timeSec, keyframeB->timeSec));
}

} // namespace d5::flex_motion::Motions
