#include "dirsig5/flex_motion/Mounts/DataDriven.h"

#include "dirsig5/base/LerpUtils.h"

namespace d5::flex_motion::Mounts {

DualQuaternion DataDriven::doGet(const DateTime &, double relativeTime,
                                 double scanFrac) const {
  if (useScanFraction)
    relativeTime = scanFrac;
  if (keyframes.empty())
    return {};
  const Keyframe &keyframe0 = keyframes.front();
  const Keyframe &keyframeN = keyframes.back();
  double timeUnits0 = keyframe0.timeUnits;
  double timeUnitsN = keyframeN.timeUnits;
  Point translation = {0, 0, 0};
  Point rotation = {0, 0, 0};
  if (relativeTime <= timeUnits0) {
    translation = {keyframe0.translation[0], keyframe0.translation[1],
                   keyframe0.translation[2]};
    rotation = {keyframe0.rotation[0], keyframe0.rotation[1],
                keyframe0.rotation[2]};
  } else if (relativeTime >= timeUnitsN) {
    translation = {keyframeN.translation[0], keyframeN.translation[1],
                   keyframeN.translation[2]};
    rotation = {keyframeN.rotation[0], keyframeN.rotation[1],
                keyframeN.rotation[2]};
  } else {
    auto [keyframeA, keyframeB] =
        findLerpIterators(keyframes.begin(), keyframes.end(), relativeTime,
                          [](const Keyframe &keyframe, double timeUnits) {
                            return keyframe.timeUnits < timeUnits;
                          });
    double fac = findLerpFactor(relativeTime, keyframeA->timeUnits,
                                keyframeB->timeUnits);
    translation = {
        (1 - fac) * keyframeA->translation[0] + fac * keyframeB->translation[0],
        (1 - fac) * keyframeA->translation[1] + fac * keyframeB->translation[1],
        (1 - fac) * keyframeA->translation[2] +
            fac * keyframeB->translation[2]};
    rotation = {
        (1 - fac) * keyframeA->rotation[0] + fac * keyframeB->rotation[0],
        (1 - fac) * keyframeA->rotation[1] + fac * keyframeB->rotation[1],
        (1 - fac) * keyframeA->rotation[2] + fac * keyframeB->rotation[2]};
  }
  return DualQuaternion(
      Quaternion(makeRotation(rotation, &rotationOrder[0]).rotation()),
      translation);
}

} // namespace d5::flex_motion::Mounts
