#include "dirsig5/flex_motion/Mounts/Lemniscate.h"

#include "dirsig5/base/Constants.h"

namespace d5::flex_motion::Mounts {

DualQuaternion Lemniscate::doGet(const DateTime &, double,
                                 double scanFrac) const {
  double acrossTrackFrac = 0.5 * (1 - std::cos(TwoPI * scanFrac));
  double alongTrackFrac = 0.5 * (1 - std::sin(4 * PI * scanFrac));
  double acrossTrackAngle =
      minAcrossTrackAngle +
      acrossTrackFrac * (maxAcrossTrackAngle - minAcrossTrackAngle);
  double alongTrackAngle =
      minAlongTrackAngle +
      alongTrackFrac * (maxAlongTrackAngle - minAlongTrackAngle);
  return DualQuaternion(
      Quaternion(Eigen::AngleAxisd(acrossTrackAngle, Vector::UnitY()) *
                 Eigen::AngleAxisd(alongTrackAngle, Vector::UnitX())),
      Point(0, 0, 0));
}

} // namespace d5::flex_motion::Mounts
