#include "dirsig5/flex_motion/RotationEngines/EulerAngles.h"

#include "dirsig5/base/LerpUtils.h"

namespace d5::flex_motion::RotationEngines {

Quaternion EulerAngles::doGet(const DateTime &, double relativeTime) const {
  if (keyframes.empty())
    return {1, 0, 0, 0};
  const Keyframe &keyframe0 = keyframes.front();
  const Keyframe &keyframeN = keyframes.back();
  double timeSec0 = keyframe0.timeSec;
  double timeSecN = keyframeN.timeSec;
  if (loop)
    relativeTime = wrapFloat(relativeTime, timeSec0, timeSecN);
  Point angles = {0, 0, 0};
  if (relativeTime <= timeSec0) {
    angles = {keyframe0.angles[0], //
              keyframe0.angles[1], //
              keyframe0.angles[2]};
  } else if (relativeTime >= timeSecN) {
    angles = {keyframeN.angles[0], //
              keyframeN.angles[1], //
              keyframeN.angles[2]};
  } else {
    auto [keyframeA, keyframeB] =
        findLerpIterators(keyframes.begin(), keyframes.end(), relativeTime,
                          [](const Keyframe &keyframe, double timeSec) {
                            return keyframe.timeSec < timeSec;
                          });
    double fac =
        findLerpFactor(relativeTime, keyframeA->timeSec, keyframeB->timeSec);
    angles = {(1 - fac) * keyframeA->angles[0] + fac * keyframeB->angles[0],
              (1 - fac) * keyframeA->angles[1] + fac * keyframeB->angles[1],
              (1 - fac) * keyframeA->angles[2] + fac * keyframeB->angles[2]};
  }
  return cache.bodyToEnuTransform *
         Quaternion(makeRotation(angles, &rotationOrder[0]).rotation());
}

} // namespace d5::flex_motion::RotationEngines
