#include "dirsig5/flex_motion/RotationEngines/Quaternions.h"

#include "dirsig5/base/LerpUtils.h"

namespace d5::flex_motion::RotationEngines {

Quaternion Quaternions::doGet(const DateTime &, double relativeTime) const {
  if (keyframes.empty())
    return cache.bodyToEnuTransform * localTransform;
  const Keyframe &keyframe0 = keyframes.front();
  const Keyframe &keyframeN = keyframes.back();
  double timeSec0 = keyframe0.timeSec;
  double timeSecN = keyframeN.timeSec;
  if (loop)
    relativeTime = wrapFloat(relativeTime, timeSec0, timeSecN);
  auto [keyframeA, keyframeB] =
      findLerpIterators(keyframes.begin(), keyframes.end(), relativeTime,
                        [](const Keyframe &keyframe, double timeSec) {
                          return keyframe.timeSec < timeSec;
                        });
  double fac =
      findLerpFactor(relativeTime, keyframeA->timeSec, keyframeB->timeSec);
  return cache.bodyToEnuTransform *
         keyframeA->quaternion.slerp(fac, keyframeB->quaternion) *
         localTransform;
}

} // namespace d5::flex_motion::RotationEngines
