#include "dirsig5/flex_motion/RotationEngines/Spin.h"

#include "dirsig5/base/LerpUtils.h"

#include <iostream>

namespace d5::flex_motion::RotationEngines {

void Spin::doUpdateState(const StateDateTime&, const geolib::Facade* facade) {
  std::sort(
    keyframes.begin(), keyframes.end(),
    [](const Keyframe& lhs, const Keyframe& rhs) {
      return lhs.timeSec < rhs.timeSec;
    });
  if (keyframes.size() > 0) {
    keyframes.front().cache.angle = 0;
    for (size_t i = 1; i < keyframes.size(); i++) {  // Integrate
      auto& keyframe0 = keyframes[i - 1];
      auto& keyframe1 = keyframes[i];
      keyframe1.cache.angle = keyframe0.cache.angle;
      keyframe1.cache.angle += 0.5 * (keyframe0.rate + keyframe1.rate) *
                               (keyframe1.timeSec - keyframe0.timeSec);
    }
  }
}

Quaternion Spin::doGet(const DateTime&, double relativeTime) const {
  if (keyframes.size() == 0) return {1, 0, 0, 0};
  const Keyframe& keyframe0 = keyframes.front();
  const Keyframe& keyframeN = keyframes.back();
  double timeSec0 = keyframe0.timeSec;
  double timeSecN = keyframeN.timeSec;
  double angle = 0.0;
  if (relativeTime < timeSec0) {
    angle = keyframe0.rate * (relativeTime - timeSec0) + keyframe0.cache.angle;
  } else if (relativeTime > timeSecN) {
    angle = keyframeN.rate * (relativeTime - timeSecN) + keyframeN.cache.angle;
  } else {
    auto [keyframeA, keyframeB] = findLerpIterators(
      keyframes.begin(), keyframes.end(), relativeTime,
      [](const Keyframe& keyframe, double timeSec) {
        return keyframe.timeSec < timeSec;
      });
    double fac =
      findLerpFactor(relativeTime, keyframeA->timeSec, keyframeB->timeSec);
    angle = ((1 - 0.5 * fac) * keyframeA->rate + 0.5 * fac * keyframeB->rate) *
              fac * (keyframeB->timeSec - keyframeA->timeSec) +
            keyframeA->cache.angle;
  }
  Vector localSpinAxis = spinAxis;
  if (frame == geolib::Location::Type::EcefType)
    localSpinAxis = cache.ecefToEnuTransform.rotation() * spinAxis;
  Quaternion rotation(Eigen::AngleAxisd(angle, localSpinAxis));
  return cache.bodyToEnuTransform * rotation;
}

}  // namespace d5::flex_motion::RotationEngines
