#include "dirsig5/flex_motion/RotationEngines/Velocity.h"

namespace d5::flex_motion::RotationEngines {

Quaternion Velocity::doGet(const DateTime &absoluteTime, double) const {
  auto velocity = locationEngine->get(absoluteTime).velocity;
  DIRSIG_THROW_IF((velocity[0] == 0 && velocity[1] == 0 && velocity[2] == 0),
                  "Using velocity for orientation but velocity was zero!");

  auto cVector = constraintVector;
  if ((constraintFlags & VectorFrameEnu) == VectorFrameEnu) {
    if (frame != d5::geolib::Location::EnuType) {
      cVector = cache.ecefToEnuTransform.inverse() * constraintVector;
    }
  } else if ((constraintFlags & VectorFrameEcef) == VectorFrameEcef) {
    if (frame != d5::geolib::Location::EcefType) {
      cVector = cache.ecefToEnuTransform * constraintVector;
    }
  }
  Vector yVector = velocity.normalized();
  Vector xVector = yVector.cross(cVector).normalized();
  Vector zVector = xVector.cross(yVector).normalized();
  Matrix matrix = Matrix::Identity();
  for (int i = 0; i < 3; i++) {
    matrix(i, 0) = xVector(i);
    matrix(i, 1) = yVector(i);
    matrix(i, 2) = zVector(i);
  }
  return Quaternion(matrix.rotation());
}

} // namespace d5::flex_motion::RotationEngines
