/** Interface file for the LocalOutputFileDriver class
 *
 *  \file LocalOutputFileDriver.h
 */

#pragma once

#include <fstream>

#include "dirsig5/io/OutputFileDriver.h"

namespace d5::io {

class LocalOutputFileDriver : public OutputFileDriver {
public:
  ~LocalOutputFileDriver() override = default;

  [[nodiscard]] bool is_open() const { return ofs_.is_open(); }

  bool seekp(std::size_t pos) {
    if (!EnsureIsOpen()) return false;
    ofs_.seekp(pos); /* NOLINT (Apple seems to use the wrong type for pos) */
    return ofs_.good();
  }

  Callbacks getCallbacks() override;

private:
  bool EnsureIsOpen() {
    if (!is_open()) {
      lastError_ = "no file is open, call open() first";
      return false;
    }
    return true;
  }

private:
  std::ofstream ofs_;
  std::string lastError_;
};

extern "C" OutputFileDriver* MakeLocalOutputFileDriver();
}  // namespace d5::io
