#include "dirsig5/io/image/ImageOutputFile.h"

#include <limits>

#include "dirsig5/base/StringUtils.h"
#include "dirsig5/io/image/EnviOutputFile.h"

namespace d5::io {

double *ImageOutputFile::setupBuffer(std::size_t pixelCount) {
  // get the number of bands
  auto bandCount = getValue<bands>();

  // check if we can reuse the buffer
  if (this->pixelBufferSize == (pixelCount * bandCount)) {
    // buffer is the same size, so reuse it
    return this->pixelBuffer.get();
  }

  // compute the size of the buffer (pixels x bands)
  this->pixelBufferSize = pixelCount * bandCount;
  this->pixelBuffer = std::make_unique<double[]>(this->pixelBufferSize);

  return this->pixelBuffer.get();
}

ImageOutputFile *ImageOutputFile::create(const std::string &name) {
  auto nameLc = d5::toLower(name);
  if (nameLc == "envi") {
    return new EnviOutputFile();
  }
  return nullptr;
}
} // namespace d5::io
