#pragma push_macro("emit")
#undef emit
#include <tbb/parallel_for.h>
#pragma pop_macro("emit")

#include "dirsig5/raster/Raster.h"

namespace d5::raster {

void Raster::create(size_t rows, size_t cols, size_t channels, DType type,
                    size_t stride) {
  if (stride == 0) {
    stride = cols * channels * ByteSize(type);

    // We used to 4-byte align for some reason ... this seems to create more
    // problems than it solves. People tend to assume data is packed if/when
    // computing indexes manually.
    /* size_t align = ByteSize(type) <= 4 ? 32 : 64;
       size_t remainder = stride % align;
       if (remainder) stride += align - remainder; */
  }
  // create the buffer and update the sizes
  type_ = type;
  rows_ = rows, rowStride_ = stride;
  cols_ = cols;
  channels_ = channels;
  channelMetadata_.clear();
  channelMetadata_.resize(channels);
  buffer_.reset(new uint8_t[rows_ * rowStride_],
                [](uint8_t *ptr) { delete[] ptr; });
}

void Raster::flip(bool flipX, bool flipY) {
  size_t halfX = cols() / 2;
  size_t halfY = rows() / 2;
  if (flipX)
    dispatch([&](auto im) {
      tbb::parallel_for(size_t{0}, size_t{im.rows()}, [&](size_t r) {
        for (size_t c = 0; c < halfX; c++) {
          auto p0 = im.pixel(r, c);
          auto p1 = im.pixel(r, im.cols() - c - 1);
          for (size_t b = 0; b < im.channels(); b++)
            std::swap(p0[b], p1[b]);
        }
      });
    });
  if (flipY)
    dispatch([&](auto im) {
      tbb::parallel_for(size_t{0}, size_t{im.cols()}, [&](size_t c) {
        for (size_t r = 0; r < halfY; r++) {
          auto p0 = im.pixel(r, c);
          auto p1 = im.pixel(im.rows() - r - 1, c);
          for (size_t b = 0; b < im.channels(); b++)
            std::swap(p0[b], p1[b]);
        }
      });
    });
}

template class RasterT<uint8_t>;
template class RasterT<uint16_t>;
template class RasterT<uint32_t>;
template class RasterT<uint64_t>;
template class RasterT<int16_t>;
template class RasterT<int32_t>;
template class RasterT<int64_t>;
template class RasterT<float>;
template class RasterT<double>;

} // namespace d5::raster
